def atno(at):
    """
    Returns the atom number of the element.

            Parameters:
                    at (char): Symbol of the element

            Returns:
                    atno (int): Atom number of the element
    """
    at_num_dict ={"H": 1,"He": 2,"Li": 3,"Be": 4,"B": 5,"C": 6,"N": 7,"O": 8,"F": 9,"Ne": 10,"Na": 11,"Mg": 12,"Al": 13,"Si": 14,"P": 15,"S": 16,"Cl": 17,"Ar": 18,"K": 19,"Ca": 20, \
        "Sc": 21,"Ti": 22,"V": 23,"Cr": 24,"Mn": 25,"Fe": 26,"Co": 27,"Ni": 28,"Cu": 29,"Zn": 30,"Ga": 31,"Ge": 32,"As": 33,"Se": 34,"Br": 35,"Kr": 36, "I":53}

    if at in at_num_dict:
        atno = at_num_dict[at]
        return(atno)
    else:
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write("Error: unknown element type encoutered in atno in module_geom.f90: " + str(at)+ " \n")




import os, sys, string
import linecache, math
import numpy as np
import datetime , time


from pople import HOF_atoms
from pople import dH_atoms
from pople import atno
from pople import uniqatoms
from pople import rung4mp2
from pople import NFC
from pople import nanb
from pople import get_geom
from pople import get_ctrl
from pople import get_val

def run():
    """ Performs thermochemistry calculations. """

    current_dir_path = os.getcwd()
    val = {}
    start_time_main = time.time()
    os.system("rm -f ORCA_G4MP2* Thermo*")

    input_file1 = current_dir_path + "/pople.inp"
    with open(input_file1, "r") as i_f:
        for line_if in i_f:
            if not line_if.startswith("#"):
                if "method_type" in line_if:
                    splt1 = line_if.split("=")
                    if "#" in splt1[1].strip():
                        new_v = splt1[1].strip().split("#")
                        val["user_method_type"] = new_v[0].strip()
                    else:
                        val["user_method_type"] = splt1[1].strip()
                if "job_type" in line_if:
                    splt1 = line_if.split("=")
                    if "#" in splt1[1].strip():
                        new_v = splt1[1].strip().split("#")
                        val["calc_choice"] = new_v[0].strip()
                    else:
                        val["calc_choice"] = splt1[1].strip()

    check_meth = ["G4MP2", "G4MP2-XP", "g4mp2", "g4mp2-xp"]
    if val["user_method_type"] not in check_meth:
       print("Invalid method_type selected, check for typos or refer manual")
       exit()

    check_choice = ["IP", "EA", "PA", "BE", "HF", "AE", "ip", "ea", "pa", "be", "hf", "ae"]
    if val["calc_choice"] not in check_choice:
       print("Invalid job_type selected, check for typos or refer manual")
       exit()


    
    if val["calc_choice"] == "IP" or val["calc_choice"] == "ip": 
        val["calc_IP"] = "true"
        val["verticalIP"] = "false"
    if val["calc_choice"] == "EA" or val["calc_choice"] == "ea": 
        val["calc_EA"] = "true"
        val["verticalEA"] = "false"
        val["calc_IP"] = "false"
    if val["calc_choice"] == "PA" or val["calc_choice"] == "pa": 
        val["calc_PA"] = "true"
        val["calc_IP"] = "false"
        val["calc_EA"] = "false"
    if val["calc_choice"] == "BE" or val["calc_choice"] == "be": 
        val["calc_BE"] = "true"
        val["calc_IP"] = "false"
        val["calc_EA"] = "false"
        val["calc_PA"] = "false"
    if val["calc_choice"] == "HF" or val["calc_choice"] == "hf": 
        val["calc_AE"] = "true"  ## check!!!
        val["calc_HF"] = "true"
        val["calc_BE"] = "false"
        val["calc_IP"] = "false"
        val["calc_EA"] = "false"
        val["calc_PA"] = "false"

    if val["calc_choice"] == "AE" or val["calc_choice"] == "ae": 
        val["calc_HF"] = "false"
        val["calc_AE"] = "true"  ## check!!!
        val["calc_BE"] = "false"
        val["calc_IP"] = "false"
        val["calc_EA"] = "false"
        val["calc_PA"] = "false"

    check_choice = ["IP", "EA", "PA", "BE", "HF", "AE", "ip", "ea", "pa", "be", "hf", "ae"]
    if val["calc_choice"] not in check_choice:
       print("Invalid job_type selected, check for typos or refer manual")


    get_geom(val["calc_choice"])
    print(val["user_method_type"], val["calc_choice"])
    new_val1 = get_ctrl(val["user_method_type"])
    #print(new_val1)
    print("checking ctrl")
    val.update(new_val1)

    #input_f = current_dir_path + "/control.inp"
    xyz_path = current_dir_path + "/geom.xyz"
    
    ### declaring units - S
    kB = 1.38064852000000*(10**-23) #kB = 1.38064852000000d-23
    h = 6.626070040*(10**-34)    #6.626070040d-34
    Ry = 10973731.568508  #10973731.568508d0
    c = 299792458    #299792458d0
    T = 298.15  #298.15d0 
    e = 1.60217648700000*(10**-19)   #1.60217648700000d-19
    N_avo = 6.02214179000000*(10**+23)  #6.02214179000000d+23
    au2j = 2 * Ry * h * c
    
    au2ev  = 2 * Ry * h * c          / e
    au2kjm = 2 * Ry * h * c * N_avo / 1000
    au2kcm = 2 * Ry * h * c * N_avo / 4184
    kcm2au = 1 / au2kcm
    
    j2au   = 1 / au2j
    au2cmi = 2 * Ry / 100
    cmi2au = 1 / au2cmi
    ### declaring units - E
    
    ### Writing intro Thermochemistry.out - start
    with open("Thermochemistry.out", "w") as ther_chem:
        day = datetime.datetime.today().strftime("%A")
        date = datetime.date.today()
        cur_date = date.strftime("%B %d, %Y")
        now = datetime.datetime.now()
        cur_time = now.strftime(" %H:%M:%S")
    
        ther_chem.write("==============================================\n")
        ther_chem.write("|           G4MP2 THERMOCHEMISTRY            |\n")
        ther_chem.write("==============================================\n")
        ther_chem.write("Starting time:  " + day + "  "+ cur_date + "  " + cur_time + "  " + time.tzname[0] +"  \n\n")
        ther_chem.write("* Constants and conversion factors *\n\n")
        ther_chem.write("h      =   6.6260700399999999E-34 J s\n")
        ther_chem.write("kB     =   1.3806485199999999E-23 J K^-1\n")
        ther_chem.write("Ry     =   1.0973731568507999E+07 m^-1\n")
        ther_chem.write("e      =   1.6021764870000000E-19 C\n")
        ther_chem.write("c      =   2.9979245800000000E+08 m s^-1\n")
        ther_chem.write("N_avo  =   6.0221417899999999E+23 mol^-1\n\n")
    
        ther_chem.write("au2ev  =   2.7211388291762535E+01\n")
        ther_chem.write("au2kcm =   6.2750957099203276E+02\n")
        ther_chem.write("au2kjm =   2.6255000450306652E+03\n")
        ther_chem.write("au2cmi =   2.1947463137015997E+05\n\n")
        
        ther_chem.write(" * Parameterized for the following atoms  *\n")
        ther_chem.write("    ____                                    \n")
        ther_chem.write("    |H |                                    \n")
        ther_chem.write("    --------    ---------------------       \n")
        ther_chem.write("    |Li| Be|    |B  |C  |N  |O  |F  |       \n")
        ther_chem.write("    |Na| Mg|    |Al |Si |P  |S  |Cl |       \n")
        ther_chem.write("    |K | Ca|    |Ga |Ge |As |Se |Br |       \n")
        ther_chem.write("    --------    ---------------------       \n")
        ther_chem.write("\n")
    ### Writing intro Thermochemistry.out - end
    
    ### List of predefined variables ?? does the order and manner of occurance matter or can all of them be declared at the begining?
    val["ALLELE"] = "false"
    val["switch_load_rel_file"] = "false"
    val["switch_guess"] = "false"
    val["FROZEN_GEOM"] = "false"
    
    val["SO_3rdrow_mols"] = "false"
    val["iterhess"] = 5
    
    val["TcutDOPre"] = 3e-2
    
    val["HF_CBS_default"]      = "true"  ## ask!!!
    val["HF_CBS_orca_23_def2"] = "false" 
    val["HF_CBS_orca_34_def2"] = "false" 
    val["HF_CBS_orca_23_cc"]   = "false" 
    val["HF_CBS_orca_34_cc"]   = "false" 
    
    val["switch_DLPNO_CCSDT"]  = "false"
    
    val["G4MP2TM"] = "false"
    val["ccsdt_cbs"] = "false"
    val["noHLC"] = "false"
    
    val["restart_check"] = "false"
    
    val["switch_RIMP2_small"] = "false"
    val["switch_read_RIMP2_small"] = "false"

    val["verticalIP"] = "false"
    val["verticalEA"] = "false"
    
    ### Reading input settings from input_f - start
    new_val = get_val()
    val.update(new_val)
    #print(val)
   
    untouc_env = os.environ['PATH'] 
    eline1 =  val["openmpi_dir"] + "bin:"
    new_env1 = eline1 + untouc_env
    os.environ['PATH'] = new_env1

    env2 = os.environ['LD_LIBRARY_PATH']
    eline2 =  val["openmpi_dir"] + "lib:"
    new_env2 = eline2 + env2
    os.environ['LD_LIBRARY_PATH'] = new_env2
    
    env3 = os.environ['LD_LIBRARY_PATH']
    eline3 = "/apps/lib:"
    new_env3 = eline3 + env3
    os.environ['LD_LIBRARY_PATH'] = new_env3
    #os.system(eline1)
    #os.system(eline2)
    #os.system(eline3)
    print(eline1)
    print(eline2)
    print(eline3)
#export PATH=/apps/openmpi-3.0.0_install/bin:$PATH
#export LD_LIBRARY_PATH=/apps/openmpi-3.0.0_install/lib:$LD_LIBRARY_PATH
#export LD_LIBRARY_PATH=/apps/lib:$LD_LIBRARY_PATH


    with open("geom.xyz","r") as xyz_f:
        num_l_xyz = sum(1 for l in xyz_f)
        Nat = int(linecache.getline("geom.xyz",1).strip())
        l1_chr_mul = linecache.getline("geom.xyz",2)
        tria = l1_chr_mul.split()
        charge = int(tria[0])
        multip = int(tria[1])
        sym = []   # will contain list of atom symbols in the mol # same order
        Rlist = []
        sym_coord = []
        for tmp_l in range(3,num_l_xyz+1):   ### what does NH do ???
            l1_xyz_1 = linecache.getline("geom.xyz",tmp_l)
            l1_lsp1 = l1_xyz_1.split()
            #full_l = l1_xyz_1.strip()
            full_l1 = l1_xyz_1.strip().split()
            full_l = full_l1[0] + "     "+str(round(float(full_l1[1]),8)) + "     "+str(round(float(full_l1[2]),8)) + "     "+str(round(float(full_l1[3]),8))
            sym_coord.append(full_l)  ## contains atom and coords
            sym.append(l1_lsp1[0])
            Rlist.append(l1_lsp1[1])
            Rlist.append(l1_lsp1[2])
            Rlist.append(l1_lsp1[3])
            conv = np.array(Rlist)
            R_coord = np.resize(conv,[Nat,3])
    
    val["Ntotal"] = 0
    val["Ntotale"] = 0
    val["Ntotalecore"] = 0
    for tmp_j in range(Nat):
        na_nb_list = nanb(sym[tmp_j])
        a = na_nb_list[0]
        b = na_nb_list[1]
        val["Ntotal"] = val["Ntotal"] + a + b
        #print(tmp_j,a,b)
        val["Ntotale"] = val["Ntotale"] + atno(sym[tmp_j])
        val["Ntotalecore"] = val["Ntotalecore"] + NFC(sym[tmp_j])
    
    
    val["Ntotal"] = val["Ntotal"] - charge
    val["Ntotale"] = val["Ntotale"] - charge
    val["Na"] = (multip + val["Ntotal"] - 1 ) / 2.0
    val["Nb"] = val["Na"] + 1 - multip
    
    
    val["isatom"] = "false"
    if Nat == 1: 
        val["isatom"] = "true"
    
    uniqat_d = uniqatoms(sym)
    
    val["IPss"] = "false"
    
    os.system("cp geom.xyz inp.xyz")
    rung4mp2(val, start_time_main)
    os.system("cp freq0.txt freq.txt")
    
    # revisit
    U0mol = val["U0"] 
    UTmol = val["UT"]
    HTmol = val["HT"]
    HLCmol = val["HLC0"]
    
     #=== JOB-TYPE-1 IS IP, from here
    if val["calc_IP"] == "true":
        val["IPss"] = "true"
        with open("geom_cation_IP.xyz", "r") as g_cat:  # need to look at this file
            with open("inp.xyz", "w") as inp_x:
                linecache.clearcache()
                Nat= int(linecache.getline("geom_cation_IP.xyz",1).strip())
                tr = linecache.getline("geom_cation_IP.xyz",2).strip()
                lsp11 = tr.split()
                charge = int(lsp11[0])
                multip = int(lsp11[1])
                inp_x.write(str(Nat) + " \n")
                inp_x.write(str(charge) +" "+ str(multip) + " \n")

                sym = []   # will contain list of atom symbols in the mol # same order
                Rlist = []
                sym_coord = []
                for tmp_l in range(3,num_l_xyz+1):   
                    l1_xyz_2 = linecache.getline("geom_cation_IP.xyz",tmp_l)
                    l1_lsp2 = l1_xyz_2.split()
                    full_l1 = l1_xyz_2.strip().split()
                    full_l = full_l1[0] + "     "+str(round(float(full_l1[1]),8)) + "     "+str(round(float(full_l1[2]),8)) + "     "+str(round(float(full_l1[3]),8))
                    sym_coord.append(full_l)  ## contains atom and coords
                    sym.append(l1_lsp2[0])
                    Rlist.append(l1_lsp2[1])
                    Rlist.append(l1_lsp2[2])
                    Rlist.append(l1_lsp2[3])
                    conv = np.array(Rlist)
                    R_coord = np.resize(conv,[Nat,3])
                for tmp_y in range(len(sym_coord)):
                    inp_x.write(sym_coord[tmp_y]) 
                    inp_x.write("\n") 
    
                val["Ntotal"] = 0
                val["Ntotale"] = 0
                val["Ntotalecore"] = 0

                for tmp_k in range(Nat):
                    na_nb_l = nanb(sym[tmp_k])
                    na = na_nb_l[0]
                    nb = na_nb_l[1]
                    val["Ntotal"] = val["Ntotal"] + na + nb
                    val["Ntotale"] = val["Ntotale"] + atno(sym[tmp_k])
                    val["Ntotalecore"] = val["Ntotalecore"] + NFC(sym[tmp_k])
    
        val["Ntotal"] = val["Ntotal"] - charge
        val["Ntotale"] = val["Ntotale"] - charge
        val["Na"] = (multip + val["Ntotal"] - 1 ) / 2.0
        val["Nb"] = val["Na"] + 1 - multip
    
        #print(val["Na"], val["Nb"], val["Ntotal"], val["Ntotale"] , charge, multip, "==IP==")
    
        val["isatom"] = "false"
        if Nat == 1: val["isatom"] = "true"
    
        rung4mp2(val, start_time_main)
        os.system("cp freq0.txt freq_cation_IP.txt")
    
        U0mol2 = U0mol - val["U0"] 
        UTmol2 = UTmol - val["UT"]
        HTmol2 = HTmol - val["HT"]
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write(" *---------*\n")
            ther_chem.write(" * SUMMARY *\n")
            ther_chem.write(" *---------*\n\n")
            ther_chem.write(" * Ionization Energy (mol+ - mol) =       " + str(round(-U0mol2,8)) + " Hartree\n")
            ther_chem.write(" * Ionization Energy (mol+ - mol) =       " + str(round(-U0mol2*au2ev,8)) + " eV\n")
            ther_chem.write(" * Ionization Energy (mol+ - mol) =       " + str(round(-U0mol2*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * Ionization Energy (mol+ - mol) =       " + str(round(-U0mol2*au2kjm,8)) + " kj/mol\n\n")

            ther_chem.write("* Breakdown *\n")
            ther_chem.write(" * CCSD(T) (mol+ - mol)           =       " + str(round((val["IPbreakdown2"][0]-val["IPbreakdown1"][0])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * DE (MP2)(mol+ - mol)           =       " + str(round((val["IPbreakdown2"][2]-val["IPbreakdown1"][2])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * DE (HF) (mol+ - mol)           =       " + str(round((val["IPbreakdown2"][1]-val["IPbreakdown1"][1])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * HLC     (mol+ - mol)           =       " + str(round((val["IPbreakdown2"][3]-val["IPbreakdown1"][3])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * SO      (mol+ - mol)           =       " + str(round((val["IPbreakdown2"][4]-val["IPbreakdown1"][4])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * ZPVE    (mol+ - mol)           =       " + str(round((val["IPbreakdown2"][5]-val["IPbreakdown1"][5])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write("                                    -------------------------\n")
            ther_chem.write("TOTAL                             =       " + str(round( (sum(val["IPbreakdown2"])  -sum(val["IPbreakdown1"]))*au2kcm,8)) +  " kcal/mol\n")
            ther_chem.write("                                    -------------------------\n\n")
            end_time_n = time.time()
            ther_chem.write("** Elapsed time =              " + str(round(end_time_n - start_time_main,2)) + "  s\n")

        day = datetime.datetime.today().strftime("%A")
        date = datetime.date.today()
        cur_date = date.strftime("%B %d, %Y")
        now = datetime.datetime.now()
        cur_time = now.strftime(" %H:%M:%S")
    
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write("==============================================\n")
            ther_chem.write("Final time:  " + day + "  "+ cur_date + "  " + cur_time + "  " + time.tzname[0] +"  \n\n")
            ther_chem.write("==============================================\n")


        exit()
    
    if val["calc_EA"] == "true":
        if val["verticalEA"] == "true": val["verticalIP"] = val["verticalEA"]
        val["IPss"] = "true"
    
        with open("geom_anion_EA.xyz", "r") as g_cat:  # need to look at this file
            with open("inp.xyz", "w") as inp_x:
                linecache.clearcache()
                Nat= int(linecache.getline("geom_anion_EA.xyz",1).strip())
                tr = linecache.getline("geom_anion_EA.xyz",2).strip()
                lsp11 = tr.split()
                charge = int(lsp11[0])
                multip = int(lsp11[1])
                inp_x.write(str(Nat) + " \n")

                inp_x.write(str(charge) +" "+ str(multip) + " \n")
                sym = []   # will contain list of atom symbols in the mol # same order
                Rlist = []
                sym_coord = []
                for tmp_l in range(3,num_l_xyz+1):   
                    l1_xyz_2 = linecache.getline("geom_anion_EA.xyz",tmp_l)
                    l1_lsp2 = l1_xyz_2.split()

                    full_l1 = l1_xyz_2.strip().split()
                    full_l = full_l1[0] + "     "+str(round(float(full_l1[1]),8)) + "     "+str(round(float(full_l1[2]),8)) + "     "+str(round(float(full_l1[3]),8))
                    sym_coord.append(full_l)  ## contains atom and coords
                    sym.append(l1_lsp2[0])
                    Rlist.append(l1_lsp2[1])
                    Rlist.append(l1_lsp2[2])
                    Rlist.append(l1_lsp2[3])
                    conv = np.array(Rlist)
                    R_coord = np.resize(conv,[Nat,3])
                for tmp_y in range(len(sym_coord)):
                    inp_x.write(sym_coord[tmp_y])
                    inp_x.write("\n")
    
    
                val["Ntotal"] = 0
                val["Ntotale"] = 0
                val["Ntotalecore"] = 0

                for tmp_k in range(Nat):
                    na_nb_l = nanb(sym[tmp_k])
                    na = na_nb_l[0]
                    nb = na_nb_l[1]
                    val["Ntotal"] = val["Ntotal"] + na + nb
                    val["Ntotale"] = val["Ntotale"] + atno(sym[tmp_k])
                    val["Ntotalecore"] = val["Ntotalecore"] + NFC(sym[tmp_k])
    
        val["Ntotal"] = val["Ntotal"] - charge
        val["Ntotale"] = val["Ntotale"] - charge
        val["Na"] = (multip + val["Ntotal"] - 1 ) / 2.0
        val["Nb"] = val["Na"] + 1 - multip
    
        #print(val["Na"], val["Nb"], val["Ntotal"], val["Ntotale"] , charge, multip, "==EA==")
    
        val["isatom"] = "false"
        if Nat == 1: val["isatom"] = "true"
    
        rung4mp2(val, start_time_main)
        os.system("cp freq0.txt freq_anion_EA.txt")
    
        U0mol2 = U0mol - val["U0"]
        UTmol2 = UTmol - val["UT"]
        HTmol2 = HTmol - val["HT"]
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write(" *---------*\n")
            ther_chem.write(" * SUMMARY *\n")
            ther_chem.write(" *---------*\n\n")
            ther_chem.write(" * Electron affinity (mol+ - mol -) =       " + str(round(U0mol2,8)) + " Hartree\n")
            ther_chem.write(" * Electron affinity (mol+ - mol -) =       " + str(round(U0mol2*au2ev,8)) + " eV\n")
            ther_chem.write(" * Electron affinity (mol+ - mol -) =       " + str(round(U0mol2*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * Electron affinity (mol+ - mol -) =       " + str(round(U0mol2*au2kjm,8)) + " kj/mol\n\n")
    
            ther_chem.write("* Breakdown *\n")
            ther_chem.write(" * CCSD(T) (mol+ - mol)           =       " + str(round((val["EAbreakdown1"][0]-val["EAbreakdown2"][0])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * DE (MP2)(mol+ - mol)           =       " + str(round((val["EAbreakdown1"][2]-val["EAbreakdown2"][2])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * DE (HF) (mol+ - mol)           =       " + str(round((val["EAbreakdown1"][1]-val["EAbreakdown2"][1])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * HLC     (mol+ - mol)           =       " + str(round((val["EAbreakdown1"][3]-val["EAbreakdown2"][3])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * SO      (mol+ - mol)           =       " + str(round((val["EAbreakdown1"][4]-val["EAbreakdown2"][4])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * ZPVE    (mol+ - mol)           =       " + str(round((val["EAbreakdown1"][5]-val["EAbreakdown2"][5])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write("                                    -------------------------\n")
            ther_chem.write("TOTAL                             =       " + str(round( (sum(val["EAbreakdown1"])  -sum(val["EAbreakdown2"]))*au2kcm,8)) +  " kcal/mol\n")
            ther_chem.write("                                    -------------------------\n\n")
            end_time_n = time.time()
            ther_chem.write("** Elapsed time =              " + str(round(end_time_n - start_time_main,2)) + "  s\n")

        day = datetime.datetime.today().strftime("%A")
        date = datetime.date.today()
        cur_date = date.strftime("%B %d, %Y")
        now = datetime.datetime.now()
        cur_time = now.strftime(" %H:%M:%S")

        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write("==============================================\n")
            ther_chem.write("Final time:  " + day + "  "+ cur_date + "  " + cur_time + "  " + time.tzname[0] +"  \n\n")
            ther_chem.write("==============================================\n")


        exit()
    
    if val["calc_PA"] == "true":
        val["verticalIP"] = "false"
        val["IPss"] = "true"
    
        with open("geom_cation_PA.xyz", "r") as g_cat:  # need to look at this file
            num_l_xyz = sum(1 for l in g_cat)
            with open("inp.xyz", "w") as inp_x:
                linecache.clearcache()
                Nat= int(linecache.getline("geom_cation_PA.xyz",1).strip())
                #print("Nat in PA", Nat)
                tr = linecache.getline("geom_cation_PA.xyz",2).strip()
                lsp11 = tr.split()
                charge = int(lsp11[0])
                multip = int(lsp11[1])
                inp_x.write(str(Nat) + " \n")
                inp_x.write(str(charge) +" "+ str(multip) + " \n")
                sym = []   # will contain list of atom symbols in the mol # same order
                Rlist = []
                sym_coord = []
                for tmp_l in range(3,num_l_xyz+1):
                    l1_xyz_2 = linecache.getline("geom_cation_PA.xyz",tmp_l)
                    l1_lsp2 = l1_xyz_2.split()
                    full_l1 = l1_xyz_2.strip().split()
                    full_l = full_l1[0] + "     "+str(round(float(full_l1[1]),8)) + "     "+str(round(float(full_l1[2]),8)) + "     "+str(round(float(full_l1[3]),8))
                    sym_coord.append(full_l)  ## contains atom and coords
                    sym.append(l1_lsp2[0])
                    Rlist.append(l1_lsp2[1])
                    Rlist.append(l1_lsp2[2])
                    Rlist.append(l1_lsp2[3])
                    conv = np.array(Rlist)
                    R_coord = np.resize(conv,[Nat,3])
                for tmp_y in range(len(sym_coord)):
                    inp_x.write(sym_coord[tmp_y])
                    inp_x.write("\n")
    
        val["Ntotal"] = 0
        val["Ntotale"] = 0
        val["Ntotalecore"] = 0
        for tmp_k in range(Nat):
            na_nb_l = nanb(sym[tmp_k])
            na = na_nb_l[0]
            nb = na_nb_l[1]
            val["Ntotal"] = val["Ntotal"] + na + nb
            val["Ntotale"] = val["Ntotale"] + atno(sym[tmp_k])
            val["Ntotalecore"] = val["Ntotalecore"] + NFC(sym[tmp_k])
    
        val["Ntotal"] = val["Ntotal"] - charge
        val["Ntotale"] = val["Ntotale"] - charge
        val["Na"] = (multip + val["Ntotal"] - 1 ) / 2.0
        val["Nb"] = val["Na"] + 1 - multip
    
        val["isatom"] = "false"
        if Nat == 1: val["isatom"] = "true"
    
        uniq_atom_res = uniqatoms(sym)
    
        rung4mp2(val, start_time_main)
        os.system("cp freq0.txt freq_cation_PA.txt")
    
        U0mol2 = U0mol - val["U0"] 
        UTmol2 = UTmol - val["UT"]
        HTmol2 = HTmol - val["HT"]
    
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write(" *---------* \n")
            ther_chem.write(" * SUMMARY * \n")
            ther_chem.write(" *---------* \n\n")
    
            ther_chem.write(" * Proton affinity (mol - molH+) =       " + str(round(HTmol2,8))        + " Hartree\n")
            ther_chem.write(" * Proton affinity (mol - molH+) =       " + str(round(HTmol2*au2ev,8))   + " eV\n")
            ther_chem.write(" * Proton affinity (mol - molH+) =       " + str(round(HTmol2*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write(" * Proton affinity (mol - molH+) =       " + str(round(HTmol2*au2kjm,8)) + " kj/mol\n\n")
    
            ther_chem.write(" * Breakdown *\n")
            ther_chem.write(" * CCSD(T)         (mol - molH+) =       " + str(round((val["PAbreakdown1"][0]-val["PAbreakdown2"][0])*au2kcm,8))   + " kcal/mol\n")
            ther_chem.write(" * DE (MP2)        (mol - molH+) =       " + str(round((val["PAbreakdown1"][2]-val["PAbreakdown2"][2])*au2kcm,8))   + " kcal/mol\n")
            ther_chem.write(" * DE (HF)         (mol - molH+) =       " + str(round((val["PAbreakdown1"][1]-val["PAbreakdown2"][1])*au2kcm,8))   + " kcal/mol\n")
            ther_chem.write(" * HLC             (mol - molH+) =       " + str(round((val["PAbreakdown1"][3]-val["PAbreakdown2"][3])*au2kcm,8))   + " kcal/mol\n")
            ther_chem.write(" * SO              (mol - molH+) =       " + str(round((val["PAbreakdown1"][4]-val["PAbreakdown2"][4])*au2kcm,8))   + " kcal/mol\n")
            ther_chem.write(" * ZPVE            (mol - molH+) =       " + str(round((val["PAbreakdown1"][5]-val["PAbreakdown2"][5])*au2kcm,8))   + " kcal/mol\n")
            ther_chem.write(" * Vib-thermal     (mol - molH+) =       " + str(round((val["PAbreakdown1"][6]-val["PAbreakdown2"][6])*au2kcm,8))   + " kcal/mol\n")  # rewrite wont work
            ther_chem.write(" * Rot-thermal     (mol - molH+) =       " + str(round((val["PAbreakdown1"][7]-val["PAbreakdown2"][7])*au2kcm,8))   + " kcal/mol\n")
            ther_chem.write(" * Trans-thermal   (mol - molH+) =       " + str(round((val["PAbreakdown1"][8]-val["PAbreakdown2"][8])*au2kcm,8))   + " kcal/mol\n")
            ther_chem.write(" * kBT             (mol - molH+) =       " + str(round((val["PAbreakdown1"][9]-val["PAbreakdown2"][9])*au2kcm,8)) + " kcal/mol\n")
            ther_chem.write("                                   -------------------------\n")
            ther_chem.write("TOTAL                             =       " + str(round( (sum(val["PAbreakdown1"])  -sum(val["PAbreakdown2"]))*au2kcm,8)) +  " kcal/mol\n")        
            ther_chem.write("                                   -------------------------\n\n")
            end_time_n = time.time()
            ther_chem.write(" ** Elapsed time =              " + str(round(end_time_n - start_time_main,2)) + " s\n\n")

        day = datetime.datetime.today().strftime("%A")
        date = datetime.date.today()
        cur_date = date.strftime("%B %d, %Y")
        now = datetime.datetime.now()
        cur_time = now.strftime(" %H:%M:%S")

        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write("==============================================\n")
            ther_chem.write("Final time:  " + day + "  "+ cur_date + "  " + cur_time + "  " + time.tzname[0] +"  \n\n")
            ther_chem.write("==============================================\n")

        exit() 
    
    
    if val["calc_BE"] == "true":
        val["verticalIP"] = "false"
        val["IPss"] = "true"
    
        with open("geom_monomer_A.xyz", "r") as g_cat:  # need to look at this file
            num_l_xyz = sum(1 for l in g_cat)
            with open("inp.xyz", "w") as inp_x:
                linecache.clearcache()
                Nat= int(linecache.getline("geom_monomer_A.xyz",1).strip())
                #print("Nat in PA", Nat)
                tr = linecache.getline("geom_monomer_A.xyz",2).strip()
                lsp11 = tr.split()
                charge = int(lsp11[0])
                multip = int(lsp11[1])
                inp_x.write(str(Nat) + " \n")
                inp_x.write(str(charge) +" "+ str(multip) + " \n")
                sym = []   # will contain list of atom symbols in the mol # same order
                Rlist = []
                sym_coord = []
                for tmp_l in range(3,num_l_xyz+1):
                    l1_xyz_2 = linecache.getline("geom_monomer_A.xyz",tmp_l)
                    l1_lsp2 = l1_xyz_2.split()
                    full_l1 = l1_xyz_2.strip().split()
                    full_l = full_l1[0] + "     "+str(round(float(full_l1[1]),8)) + "     "+str(round(float(full_l1[2]),8)) + "     "+str(round(float(full_l1[3]),8))
                    sym_coord.append(full_l)  ## contains atom and coords
                    sym.append(l1_lsp2[0])
                    Rlist.append(l1_lsp2[1])
                    Rlist.append(l1_lsp2[2])
                    Rlist.append(l1_lsp2[3])
                    conv = np.array(Rlist)
                    R_coord = np.resize(conv,[Nat,3])
                for tmp_y in range(len(sym_coord)):
                    inp_x.write(sym_coord[tmp_y])
                    inp_x.write("\n")
    
    
        val["Ntotal"] = 0
        val["Ntotale"] = 0
        val["Ntotalecore"] = 0
        for tmp_k in range(Nat):
            na_nb_l = nanb(sym[tmp_k])
            na = na_nb_l[0]
            nb = na_nb_l[1]
            val["Ntotal"] = val["Ntotal"] + na + nb
            val["Ntotale"] = val["Ntotale"] + atno(sym[tmp_k])
            val["Ntotalecore"] = val["Ntotalecore"] + NFC(sym[tmp_k])
    
        val["Ntotal"] = val["Ntotal"] - charge
        val["Ntotale"] = val["Ntotale"] - charge
        val["Na"] = (multip + val["Ntotal"] - 1 ) / 2.0
        val["Nb"] = val["Na"] + 1 - multip
    
        val["isatom"] = "false"
        if Nat == 1: val["isatom"] = "true"
    
        uniq_atom_res = uniqatoms(sym)
    
        rung4mp2(val, start_time_main)
        os.system("cp freq0.txt freq_monomer_A.txt")
    
        U0mol2 = U0mol - (2*val["U0"])
        UTmol2 = UTmol - (2*val["UT"])
        HTmol2 = HTmol - (2*val["HT"])
    
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write(" *---------* \n")
            ther_chem.write(" * SUMMARY * \n")
            ther_chem.write(" *---------* \n\n")
            
            ther_chem.write(" * Binding energy (A2 - 2 * A) = " +  str(U0mol2)        + " Hartree\n")
            ther_chem.write(" * Binding energy (A2 - 2 * A) = " +  str(U0mol2*au2ev)  + " eV\n")
            ther_chem.write(" * Binding energy (A2 - 2 * A) = " +  str(U0mol2*au2kcm) + " kcal/mol\n")
            ther_chem.write(" * Binding energy (A2 - 2 * A) = " +  str(U0mol2*au2kjm) + " kj/mol\n\n")
            end_time_n = time.time() 
            ther_chem.write(" ** Elapsed time =              " + str(round(end_time_n - start_time_main,2)) + " s\n\n")  # which end time!! ??

        day = datetime.datetime.today().strftime("%A")
        date = datetime.date.today()
        cur_date = date.strftime("%B %d, %Y")
        now = datetime.datetime.now()
        cur_time = now.strftime(" %H:%M:%S")

        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write("==============================================\n")
            ther_chem.write("Final time:  " + day + "  "+ cur_date + "  " + cur_time + "  " + time.tzname[0] +"  \n\n")
            ther_chem.write("==============================================\n")

        exit()    
    # !=== Atomic calculations
    
    
    if val["calc_AE"] == "true":
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write(" * G4MP2 for reference atoms*\n\n")
    
        U0molAE = U0mol
        UTmolAE = UTmol
        HTmolAE = HTmol
        uniq_atom_res = uniqatoms(sym)
        for tmp_o in range(uniq_atom_res["N_ua"]):
            Nat = 1
            # deallocate(sym, R)
            sym =[]
            ua = uniq_atom_res["uniq_sym"]
            if Nat == 1: val["isatom"] = "true"
            sym.append(ua[tmp_o])
            with open("Thermochemistry.out", "a") as ther_chem:
                ther_chem.write(" * ATOM:" + str(sym) + "\n")
            
            na_nb_l = nanb(sym[0])   # check
            na = na_nb_l[0]
            nb = na_nb_l[1]
            R = 0 ## ask
            multip = na - nb + 1
            charge = 0
            with open("inp.xyz", "w") as new_in:
                new_in.write(str(Nat) + "\n")
                new_in.write("0  " + str(multip) + "\n")
                new_in.write(str(sym[0]) + "  0.0 0.0 0.0 \n")  # ask
    
            na_nb_l = nanb(sym[0])
            na = na_nb_l[0]
            nb = na_nb_l[1]
            val["Ntotal"] = na + nb
    
            val["Ntotale"] = atno(ua[tmp_o])
            val["Ntotalecore"] = NFC(ua[tmp_o])
            val["Na"] = (multip + val["Ntotal"] - 1 ) / 2.0
            val["Nb"] = val["Na"] + 1 - multip
    
            #print(tmp_o,Nat,ua,val["isatom"],"=== check Nat, isatom ===")
            rung4mp2(val,start_time_main)
            uan_l = uniq_atom_res["uan"]
            U0molAE = U0molAE - uan_l[tmp_o] * val["U0"] 
            UTmolAE = UTmolAE - uan_l[tmp_o] * val["UT"]
            HTmolAE = HTmolAE - uan_l[tmp_o] * val["HT"]
            HLCmol =  HLCmol -  uan_l[tmp_o] * val["HLC0"]
    
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write(" *---------* \n")
            ther_chem.write(" * SUMMARY * \n")
            ther_chem.write(" *---------* \n\n")
    
            ther_chem.write(" * Atomization energy (Atoms - mol) = " + str(-U0molAE)  + " Hartree\n")
            ther_chem.write(" * Atomization energy (Atoms - mol) = " + str(-U0molAE*au2ev) + " eV\n")
            ther_chem.write(" * Atomization energy (Atoms - mol) = " + str(-U0molAE*au2kcm) +  " kcal/mol\n")
            ther_chem.write(" * Atomization energy (Atoms - mol) = " + str(-U0molAE*au2kjm) + " kj/mol\n")
            ther_chem.write(" * Atomization energy (Atoms - mol), NO HLC = " + str(-(U0molAE-HLCmol)*au2kcm) +  " kcal/mol\n\n")
            end_time_n = time.time() 
            ther_chem.write(" ** Elapsed time =              " + str(round(end_time_n - start_time_main,2)) + " s\n\n")  # which end time!! ??
    if val["calc_HF"] != "true":
        day = datetime.datetime.today().strftime("%A")
        date = datetime.date.today()
        cur_date = date.strftime("%B %d, %Y")
        now = datetime.datetime.now()
        cur_time = now.strftime(" %H:%M:%S")
    
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write("==============================================\n")
            ther_chem.write("Final time:  " + day + "  "+ cur_date + "  " + cur_time + "  " + time.tzname[0] +"  \n\n")
            ther_chem.write("==============================================\n")

        exit()

    if val["calc_HF"] == "true":
        val["calc_AE"] = "true"
        HOF = HTmol - U0mol + U0molAE
        HOF_0K = []
        dH_298K_0K = []
        ua = uniq_atom_res["uniq_sym"]
        for tmp_u in range(uniq_atom_res["N_ua"]):
            HOF_0K.append(HOF_atoms(ua[tmp_u]))
            dH_298K_0K.append(dH_atoms(ua[tmp_u]))

        uan_l = uniq_atom_res["uan"]
        HOF = HTmol - U0mol + U0molAE
        for tmp_u in range(uniq_atom_res["N_ua"]):
            HOF = HOF + uan_l[tmp_u] * ( HOF_0K[tmp_u] - dH_298K_0K[tmp_u] )

        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write(" *---------* \n")
            ther_chem.write(" * SUMMARY * \n")
            ther_chem.write(" *---------* \n\n")

            ther_chem.write("* Heat of formation = " + str(HOF) + " Hartree\n")
            ther_chem.write("* Heat of formation = " + str(HOF*au2ev) + " eV\n")
            ther_chem.write("* Heat of formation = " + str(HOF*au2kcm) + " kcal/mol\n")
            ther_chem.write("* Heat of formation = " + str(HOF*au2kjm) + " kj/mol\n")
            ther_chem.write("* Heat of formation, NO HLC = " +  str((HOF-HLCmol)*au2kcm) + " kcal/mol\n\n")
            end_time_n = time.time()
            ther_chem.write("** Elapsed time =              " +str(round(end_time_n - start_time_main,2)) + " s\n\n")  # which end time!! ??


    day = datetime.datetime.today().strftime("%A")
    date = datetime.date.today()
    cur_date = date.strftime("%B %d, %Y")
    now = datetime.datetime.now()
    cur_time = now.strftime(" %H:%M:%S")

    with open("Thermochemistry.out", "a") as ther_chem:
        ther_chem.write("==============================================\n")
        ther_chem.write("Final time:  " + day + "  "+ cur_date + "  " + cur_time + "  " + time.tzname[0] +"  \n\n")
        ther_chem.write("==============================================\n")

 



import os, sys, string
import linecache, math
import numpy as np
import datetime , time


from pople import NFC
from pople import uniqatoms
from pople import printbas


####### runorca - S
def runorca(method, basis,optfreq,custombasis, correlated, values):
    """
    Runs orca

            Parameters:
                    method (char) : Name of functional to be used
                    basis (char)  :  Basis set name
                    optfreq (char) : true/false value of the optfreq keyword in control.inp
                    custombasis (char) : true/false value of the custombasis keyword in control.inp
                    correlated (char) : true/false value of the coreleated keyword in control.inp
                    values (dict): Values of the control variables mentioned by user in control.inp 

    """

    with open("input.com", "w") as com_f:
        if optfreq == "true":
            if values["verticalIP"] != "true" or values["IPss"] != "true":   # IPss not defined
                if values["MGGA"] == "true":
                    Freqstr="NumFreq"
                else:
                    Freqstr="Freq"
        
                if custombasis == "true":
                    com_f.write("! " +str(method) + "  " + values["String_Opt"] + "  " + Freqstr + "  \n")
                else:
                    com_f.write("! " +str(method) + "  " + str(basis) +"   "+values["String_Opt"] + "  " + Freqstr + "  \n")
            else:
                if custombasis == "true":
                    com_f.write("! " +str(method) + "  " + Freqstr + "  \n")
                else:
                    com_f.write("! " +str(method) + "  " + str(basis) + "  " + Freqstr + "  \n")
        else:
            if custombasis == "true":
                com_f.write("! " +str(method) + "  \n")
            else:
                com_f.write("! " +str(method) + "  " + str(basis) + "  \n")

        with open("inp.xyz","r") as xyz_f:
            num_l_xyz = sum(1 for l in xyz_f)
            linecache.clearcache()
            Nat = int(linecache.getline("inp.xyz",1).strip())
            l1_chr_mul = linecache.getline("inp.xyz",2)
            if values["restart_check"] == "true":
                 test_l = "* xyz  " + str(values["restart_charge"]) + "  " + str(multip) + " \n"
            else:
                 test_l = "* xyz  " + l1_chr_mul.strip() + " \n"
            com_f.write(test_l)
            sym = []   # will contain list of atom symbols in the mol # same order
            
            for tmp_l in range(3,num_l_xyz+1):   ### what does NH do ???
                l1_xyz_1 = linecache.getline("inp.xyz",tmp_l)
                l1_lsp1 = l1_xyz_1.split()
                sym.append(l1_lsp1[0])
                com_f.write(l1_xyz_1)

        com_f.write("*\n")
        com_f.write("%MaxCore  " + values["maxcore_mb"] + "\n")
        com_f.write("%scf\n   MaxIter 500 \n")
        com_f.write("   Convergence  " + values["conv_scf"] + "\n")
        com_f.write("end\n")

        if values["switch_guess"] == "true":   ### this is not part of the inp file!!!
            if values["guess_TM"] == "true" and values["G4MP2TM"]:
                com_f.write("   Guess = " + values["option_guess"] + "\n")
                com_f.write("end\n")
        
        if values["switch_load_rel_file"] == "true":
            ### line 1499  IMPORTANT !!!!!!!!!!!!!
            f1 = open("rel_file.txt", "r")
            com_f.write(f1.read())
            f1.close()
#            print("check if rel_file.txt exists!!")
            with open("Thermochemistry.out", "a") as ther_chem:
                ther_chem.write("check if rel_file.txt exists!!")

        if values["SCFDIIS"] == "true":
            com_f.write("%scf\n  DIISMaxEq 15\n")
            com_f.write("  directresetfreq 1\n")
            com_f.write("end\n")

        if values["LSHIFT"] == "true":
            com_f.write("%scf\n")
            com_f.write("  Shift Shift 0.1 ErrOff 0.1 end\n")
            com_f.write("end\n")

        if values["SOSCF"] == "true":
            com_f.write("%scf\n")
            com_f.write("  soscfmaxit 12\n")
            com_f.write("  directresetfreq 1\n")
            com_f.write("end\n")
        if values["switch_DLPNO_CCSDT"] == "true":
          com_f.write("%mdci\n")
          com_f.write("  UseFullLMP2Guess true\n")
          com_f.write("  TcutDOPre =  " + str(values["TcutDOPre"]) +"\n")
          com_f.write("end\n")
        ### revisit for datatype check!!!!!! IMPORTANT
        if ( float(values["Ntotale"]) <= float(values["nproc"]) ) or ( (float(values["Ntotale"])-float(values["Ntotalecore"]))  < float(values["nproc"]) ):
            com_f.write("%pal nprocs  1 \n")
        else:
            com_f.write("%pal nprocs  "+values["nproc"]+" \n")
        
        com_f.write("end\n")
        com_f.write("%method\n")  ## CHECK
        com_f.write("  IntAcc 7.0\n")
        
        if values["optdiis"] == "true":
            com_f.write("  Z_solver DIIS\n")
            com_f.write("  Z_MaxIter 300\n")
        
        if correlated == "true":
            uniq_atom_res = uniqatoms(sym)
            if values["ALLELE"] == "true":  ### CHECK!!!!
                for iat in range(int(uniq_atom_res["N_ua"])):
                    pre1 = uniq_atom_res["uniq_sym"]
                    at_pr1 = pre1[iat]
                    com_f.write("  NewNCore " + at_pr1 + "  " + " 0  end\n")
            else:
                for iat in range(int(uniq_atom_res["N_ua"])):
                    pre1 = uniq_atom_res["uniq_sym"]
                    at_pr1 = pre1[iat]
                    NFC_res = NFC(at_pr1)
                    com_f.write("  NewNCore " + at_pr1 + "  " + str(NFC_res) +"  end\n")
        
        com_f.write("end\n")
        
        if optfreq == "true":
            com_f.write("%geom\n")
            if values["MGGA"] == "true":
                com_f.write("  Calc_Hess true; NumHess true\n")
            else:
                com_f.write("  Calc_Hess true\n")
            com_f.write("  Recalc_Hess " + str(values["iterhess"]) +" \n")  ## revisit !!!! CHECK!!! IMPORTANT
            com_f.write("end\n")
            com_f.write("%freq  Temp  273.15, 298.15\n")
            com_f.write("end\n")
            #com_f.write("end")
        if custombasis == "true":
            com_f.write("%basis \n")
        
    if custombasis == "true":
        with open("input.com", "a") as com_f:
            uniq_atom_res = uniqatoms(sym)
            fname =  basis
            if Nat == 1:  # check datatype
                printbas(fname, sym[0])    ## what is fname, it should be the basis set keyword. but which one??
            else:
                for iat1 in range(int(uniq_atom_res["N_ua"])):
                    pre2 = uniq_atom_res["uniq_sym"]
                    at_pr2 = pre2[iat1]
                    printbas(fname, at_pr2)
            com_f.write("end\n")
#   os.system("cat basis.com >> input.com")

    print(values["orca_dir"] + "orca input.com > input.out")
    os.system(values["orca_dir"] + "orca input.com > input.out")
    os.system("cat input.com >> ORCA_G4MP2.com")
    os.system("cat input.out >> ORCA_G4MP2.out")
####### runorca - E



def uniqatoms(sym):  # to fine uniq atoms and the number of occurance of uniq atoms in the molecule
    """
    Returns the number of unique atoms in the molecule, along with their symbols and number of occurances

            Parameters:
                    sym (list, char): List of atoms in the molecule

            Returns:
                    uniqat_d (dict): Details of number of unique atoms in the molecule
                    uniqat_d["N_ua"] (int) : Number of unique atom types in the molecule
                    uniqat_d["uniq_sym"] (list, char) : List of symbols of the unique atoms present in the molecule
                    uniqat_d["uan"] (list, int) : List of number of occurances of the unique atoms present
    """

# we want 'ua' unique atom types
# we want 'uan' no. of unique atoms of each type
# Ex. C20H42
# N_ua = 2
# ua = 'C' 'H
# uan = 20  42
    uan = []
    uniq_sym = list(set(sym))  # ua
    N_ua = len(uniq_sym)
    for tmp_i in range(len(uniq_sym)):
        count_a = 0
        for tmp_j in range(len(sym)):
            if uniq_sym[tmp_i] == sym[tmp_j]:
                count_a = count_a + 1
        uan.append(count_a)
        uniqat_d = {}
        uniqat_d["N_ua"] = int(N_ua)
        uniqat_d["uniq_sym"] = uniq_sym
        uniqat_d["uan"] = uan
        #print(uniqat_d)
    #return(N_ua, uniq_sym, uan)  ### revisit, how do you want variables returned?
    return(uniqat_d)



def HOF_atoms(at):
    """
    Returns the heat of formation of the element.

            Parameters:
                    at (char): Symbol of the element

            Returns:
                    HOF_atoms (float): Heat of formation of the element
    """

    h = 6.626070040*(10**-34)    #6.626070040d-34
    Ry = 10973731.568508  #10973731.568508d0
    c = 299792458    #299792458d0
    N_avo = 6.02214179000000*(10**+23)  #6.02214179000000d+23
    au2kcm = 2 * Ry * h * c * N_avo / 4184
    kcm2au = 1 / au2kcm

    hof_dict_1 = {"H": 51.63,"Li": 37.69,"Be": 76.48,"B": 136.2,"C": 169.98,"N": 112.53,"O": 58.99 ,"F": 18.47,"Na": 25.69,"Mg": 34.87,"Al": 78.23,"Si": 106.6,"P": 75.42,"S": 65.66,"Cl": 28.59, \
        "K": 21.48303059,"Ca": 42.38503824,"Fe": 98.7373,"Ga": 64.763384321,"Ge": 89.354,"As": 68.86,"Se": 57.8991,"Br": 28.1836,"I": 25.62}
#case ('K ') # todo collect reference for 3-rd row from Sambit
# case ('Br') !Phys. Chem. Chem. Phys., 2015, 17, 3584--3598
# case ('I') ! JANAF
# many other cases have numbers commented out...take a look at OG f90 file, thanks
    if at in hof_dict_1:
        HOF_atoms = hof_dict_1[at] * kcm2au
        return(HOF_atoms)
    else: # equivalent to case default
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write("Error: unknown element type encoutered in HOF_atoms: " + str(at)+ " \n")




from  .atno             import  atno
from  .At_SO            import  At_SO
from  .dH_atoms         import  dH_atoms
from  .HOF_atoms        import  HOF_atoms
from  .Mol_SO           import  Mol_SO
from  .nanb             import  nanb
from  .sym2mass         import  sym2mass
from  .NFC              import  NFC
from  .principal_coord  import  principal_coord
from  .printbas         import  printbas
from  .uniqatoms        import  uniqatoms
from  .rung4mp2         import  rung4mp2
from  .runorca          import  runorca 
from  .get_geom         import get_geom
from  .get_ctrl         import get_ctrl
from  .get_val          import get_val
from  .run              import  run

#print("LOADED REQUIREMENTS FOR POPLE2")



import linecache, os, sys

def get_val():
    """ Extracts control parameters from pople.inp """
    current_dir_path = os.getcwd()

    input_file = current_dir_path + "/pople.inp"

    inp_par = ["openmpi_dir","orca_dir","install_dir","maxcore_mb","nproc","method_opt_freq", "basis_opt_freq", "custombasis_opt_freq", "String_Opt", "MGGA", "FROZEN_GEOM", "method_ccsdt", \
               "basis_ccsdt", "custombasis_ccsdt", "DLPNO_CCSDT", "method_mp2_s", "basis_mp2_s", "custombasis_mp2_s", "method_mp2", "basis_mp2", "custombasis_mp2", "flag_RIMP2", \
               "flag_DLPNOMP2", "method_hf3", "basis_hf3", "custombasis_hf3", "method_hf4", "basis_hf4", "custombasis_hf4", "scalfac", "calc_IP", "verticalIP", "calc_EA", \
               "verticalEA", "calc_PA", "calc_AE", "calc_BE", "calc_HF", "conv_scf", "HLCeqZERO", "SOSCF", "SCFDIIS", "SO_3rdrow_mols", "LSHIFT", "optdiis", "HF_CBS_default", \
               "HF_CBS_orca_23_def2",  "HF_CBS_orca_34_def2", "HF_CBS_orca_23_cc", "HF_CBS_orca_34_cc", "restart_cc", "restart_mp2", "restart_hf3", "restart_hf4",  \
               "switch_RIMP2_small", "switch_read_RIMP2_small", "AA", "BB", "CC", "DD" , "EE", "ApAp", "method_type", "job_type"]
    val = {}

    with open(input_file,"r") as ini_inp:
        for line in ini_inp:
            if not line.startswith("#"):
                if "=" in line:
                    val_line = line.split("=")
                    val_key = val_line[0].strip()
                    if val_key.strip() in inp_par:
                        if "#" in val_line[1]: ### CHECK!!!!!!
                            val_split = val_line[1].split("#")   ## this condition is to make sure if some comments are added to the input line, the comment (that follows # ) is ignored
                            if ( val_split[0].strip() == "True" ) or (val_split[0].strip() == "TRUE") or (val_split[0].strip() == "T") or (val_split[0].strip() == ".true." ) or (val_split[0].strip() == "t"):
                                 val[val_key] = "true"
                            elif ( val_split[0].strip() == "False" ) or (val_split[0].strip() == "FALSE") or (val_split[0].strip() == "F") or (val_split[0].strip() == ".false.") or (val_split[0].strip() == "t"):
                                val[val_key] = "false"
                            else:
                                 val[val_key] = val_split[0].strip()
                        else:
                           if ( val_line[1].strip() == "True" ) or ( val_line[1].strip() == "TRUE") or ( val_line[1].strip() == "T") or ( val_line[1].strip() == ".true.") or ( val_line[1].strip() == "t"):
                                val[val_key] = "true"
                           elif ( val_line[1].strip() == "False" ) or (val_line[1].strip() == "FALSE") or (val_line[1].strip() == "F") or (val_line[1].strip() == ".false.") or (val_line[1].strip() == "f"):
                               val[val_key] = "false"
                           else:
                                val[val_key] = val_line[1].strip()


                    else: 
                
                        print("Keyword not in the list of parameters, please check for typos")
                        print("List of valid keywords are:")
                        for l in range(len(inp_par)):
                            print(inp_par[l])

    return(val)






import linecache, os, sys

def get_geom(setting):
    """
    Creates geometry files for pople run.
            Parameters:
                    setting (char): Type of calculation requested.

    """
    current_dir_path = os.getcwd()
    input_file = current_dir_path + "/pople.inp"
# find line no of start of geom.xyz and second geom file(is enabled)
    with open(input_file, "r") as inp_f:
        for num, line in enumerate(inp_f,1):
            if "geom_1" in line:
                geom_lno = num
            if setting == "IP" or setting == "EA" or setting == "PA" or setting == "BE":
                if "geom_2" in line:
                    sec_geom_lno = num
# creates geom.xyz
        with open("geom.xyz", "w") as g_xyz1:
            Nat = int(linecache.getline("pople.inp", geom_lno+1))
            char_mul = linecache.getline("pople.inp", geom_lno+2)
            g_xyz1.write(str(Nat) + "\n")
            g_xyz1.write(char_mul)
            for tmp_i in range(geom_lno+3, geom_lno+3+Nat):
                nl = linecache.getline("pople.inp", tmp_i)
                g_xyz1.write(nl)

#creates second geom.xyz
    if setting == "IP" or setting == "EA" or setting == "PA" or setting == "BE":
        if setting == "IP": fname2 = "geom_cation_IP.xyz"
        if setting == "EA": fname2 = "geom_anion_EA.xyz"
        if setting == "PA": fname2 = "geom_cation_PA.xyz"
        if setting == "BE": fname2 = "geom_monomer_A.xyz"
        with open(fname2, "w") as g_xyz1:
            Nat = int(linecache.getline("pople.inp", sec_geom_lno+1))
            char_mul = linecache.getline("pople.inp", sec_geom_lno+2)
            g_xyz1.write(str(Nat) + "\n")
            g_xyz1.write(char_mul)
            for tmp_i in range(sec_geom_lno+3, sec_geom_lno+3+Nat):
                nl = linecache.getline("pople.inp", tmp_i)
                g_xyz1.write(nl)



def NFC(at):   # returns no. of forzen core electrons  # Why not use elif??? revisit
    """
    Returns the number of frozen core electrons in the atom

            Parameters:
                    at (char): Symbol of the element

            Returns:
                    NHC (int): Number of frozen core electrons in the atom
    """

    if at in ['H','He']:
        return(0)
    if at in ['Li','Be','B','C','N','O','F','Ne','Na','Mg']:
        return(2)
    if at in ['Al','Si','P','S','Cl','Ar','K','Ca','Fe']:
        return(10)
    if at in ['Ga','Ge','As','Se','Br','Kr']:
        return(18)
    if at in ['I']:
        return(36)



def At_SO(at, charge):  # element symbol, charge
    """
    Returns energy correction for spin orbit coupling of the given element.

            Parameters:
                    at (char): Symbol of the element
                    charge (int): Charge on the element

            Returns:
                    At_SO (float): Energy correction for spin orbit coupling of the element
    """

    At_SO = 0
    if charge == 0:
        S_dict_0 = {"B" : -0.05, "C" : -0.14, "O" : -0.36, "F" : -0.61, "Al" : -0.34, "Si" : -0.68, "S" : -0.89, "Cl" : -1.34, "Ga" : -2.51, "Ge" : -4.41, "Se" : -4.3,  \
            "Br" : -5.6, "Fe" : -1.84, "I" : -11.548}
        if at in S_dict_0:  # this "if" executes only of the incoming atom "at" is present in the dict, else it returns 0 (this is equivalent to default in the f90 script)
            At_SO = S_dict_0[at]
            return(At_SO)
        else:
            return(At_SO)

    elif charge == +1:
        S_dict_1 = {"C"  :-0.2, "N"  :-0.43, "F"  :-0.67, "Ne" :-1.19, "Si" :-0.93, "P"  :-1.43, "Cl" :-1.68, "Ar" :-2.18, "Ge" :-5.37, "As" :-8.04, "Br" :-6.71, "Kr" :-8.16, "I"  :-14.028}
        if at in S_dict_1:  # this "if" executes only of the incoming atom "at" is present in the dict, else it returns 0 (this is equivalent to default in the f90 script)
            At_SO = S_dict_1[at]
            return(At_SO)
        else:
            return(At_SO)

    elif charge == -1:
        S_dict_n1 = {"B": -0.03, "O": -0.26, "Al": -0.28, "P": -0.45, "S": -0.88}
        if at in S_dict_n1:  # this "if" executes only of the incoming atom "at" is present in the dict, else it returns 0 (this is equivalent to default in the f90 script)
            At_SO = S_dict_n1[at]
            return(At_SO)
        else:
            return(At_SO)

    else:
        return(At_SO)




import os, sys, string
import linecache, math
import numpy as np


try:
    import importlib.resources as pkg_resources
except ImportError:
    import importlib_resources as pkg_resources


from . import templates



def printbas(fname, at):  
    """
    Prints the basis set parameters into the input file input.com

            Parameters:
                    fname (char): Basis set name
                    at (char): Symbol of the element
    """

    #bfile = pkg_resources.open_text(templates, 'GTBAS1')
    #bfile_r = pkg_resources.read_text(templates, 'GTBAS1')
    bfile_r = pkg_resources.read_text(templates, fname)
    with open("temp_bas","w") as nbfile:
        nbfile.write(bfile_r)
    #basisSet_fpath = fname
    start_phrase = "NewGTO   "+ at
    #print(start_phrase)
    num_lines_bas = sum(1 for line_tmp1 in open("temp_bas","r"))

    for temp_num, temp_l in enumerate(open("temp_bas","r")):
        if start_phrase in temp_l.strip():
            if start_phrase == temp_l.strip():
                 bas_start_lno = temp_num+1
                 break

    with open("input.com", "a") as new_f:
        for l1 in range(bas_start_lno,num_lines_bas):
            req_line_1 = linecache.getline("temp_bas", l1)
            if "end" in req_line_1.strip():
                break
            else:
                new_f.write(req_line_1)

        new_f.write("   end\n")

    os.system("rm -f temp_bas")
# fname = value of the  basis set name from the inp file,  install_dir_val = value from initial input file by user



import linecache
import numpy as np

from pople import sym2mass

def principal_coord():
    """
    Returns eigen values.
    Requires input.xyz to be present in pwd. input.xyz contains the cartesian coordinates of the molecule/radical/ion in xyz format.
    Used to characterize if the system is linear/non-linear.

            Returns:
                Ievals  (list, float): Eigen values
    """

    with open("input.xyz","r") as xyz_f:
        num_l_xyz = sum(1 for l in xyz_f)
        Nat = int(linecache.getline("input.xyz",1).strip())
        l1_chr_mul = linecache.getline("input.xyz",2)
        sym = []   # will contain list of atom symbols in the mol # same order
        Rlist = []
        for tmp_l in range(3,num_l_xyz+1):   ### what does NH do ???
            l1_xyz_1 = linecache.getline("input.xyz",tmp_l)
            l1_lsp1 = l1_xyz_1.split()
            sym.append(l1_lsp1[0])
            Rlist.append(float(l1_lsp1[1]))
            Rlist.append(float(l1_lsp1[2]))
            Rlist.append(float(l1_lsp1[3]))
        conv = np.array(Rlist)
        R_coord = np.resize(conv,[Nat,3])

        rCM_pre = []
        mass_list = []
        for tmp_j in range(Nat):
            #print(sym[tmp_j])
            mass_atom = sym2mass(sym[tmp_j])
            mass_list.append(mass_atom)
            new1 = R_coord[tmp_j] * mass_atom
            rCM_pre.append(new1)
            #rCM_pre = R_coord[tmp_j][0] + rCM_pre[0])
        rCM_sum = np.sum(rCM_pre,axis=0)
        tot_mass = sum(mass_list)
        rCM = rCM_sum/tot_mass

        new_coord1 = []
        r_skew_symm = np.zeros((3,3))
        momin = 0.0
        for tmp_k in range(Nat):
            sub_cm = R_coord[tmp_k] - rCM
            new_coord1.append(sub_cm)
            r_skew_symm[0][0] = 0.0
            r_skew_symm[0][1] = -sub_cm[2]
            r_skew_symm[0][2] = sub_cm[1]

            r_skew_symm[1][0] = sub_cm[2]
            r_skew_symm[1][1] = 0.0
            r_skew_symm[1][2] =-sub_cm[0]

            r_skew_symm[2][0] =-sub_cm[1]
            r_skew_symm[2][1] = sub_cm[0]
            r_skew_symm[2][2] = 0.0

            momin = momin + (sym2mass(sym[tmp_k]) * np.matmul(np.transpose(r_skew_symm), r_skew_symm) )

        eig_val, eig_vec = np.linalg.eig(momin)

        idx = eig_val.argsort()[::+1]
        eig_val = eig_val[idx]
        eig_vec = eig_vec[:,idx]

        Ievals = eig_val


        print("MOM== ", Ievals)
        return(Ievals)



def Mol_SO(Nat, multip, charge, sym, SO_3rdrow_mols_val):   # number of atoms, multiplicity, charge, array of atoms in molecule, value of SO_3rdrow_mols (from orca.inp file)
    """
    Returns the molecular spin orbit correction.

            Parameters:
                    Nat (int): Number of atoms in the molecule
                    multip (int): Multiplcity of the molecule
                    charge (int): Charge on the molecule
                    sym (list, char) : List of atoms in the molecule
                    SO_3rdrow_mols_val (char) : true/false value of the SO_3rdrow_mols_val keyword

            Returns:
                    Mol_SO (float): Molecular spin orbit correction
    """

    Mol_SO = 0
    
    # Special Case - Acetleyne - S
    if Nat == 4 and multip == 2 and charge == 1:
        countH_temp =0
        countC_temp =0
        for tmp in range(len(sym)):
            if sym[tmp] == "H":
                countH_temp= countH_temp +1
            if sym[tmp] == "C":
                countC_temp = countC_temp +1
        if countH_temp == 2 and countC_temp == 2:
            Mol_SO = -0.07  #-0.07d0
    
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write("DETECTED A C2H2+ SYSTEM: Using SO parameters for acetylene cation\n")
            ther_chem.write("Ref: JCP 114, 9287, 2001\n\n")
    # Special Case - Acetleyne - E
    
    # For diatomics with multip = 2
    if Nat == 2 and multip == 2 :
        sort_sym = sorted(sym, reverse=True)
        if SO_3rdrow_mols_val == "true":  # for 3rd_row elements
    
            if charge == 0:
                if sort_sym[0] == 'O' and sort_sym[1] == 'Br':  Mol_SO=-2.20
    
                # COMMMENT: paper has it for cation, but it looks like it is for neutral
                if sort_sym[0] == 'Se' and sort_sym[1] == 'H':  Mol_SO=-4.21
    
            if charge == +1:  
                if sort_sym[0] == 'K' and sort_sym[1] == 'Br':  Mol_SO=-2.99
                if sort_sym[0] == 'H' and sort_sym[1] == 'As':  Mol_SO=-3.54
                if sort_sym[0] == 'H' and sort_sym[1] == 'Br':  Mol_SO=-6.26
                if sort_sym[0] == 'F' and sort_sym[1] == 'Br':  Mol_SO=-6.10
                if sort_sym[0] == 'Na' and sort_sym[1] == 'Br':  Mol_SO=-3.93
                if sort_sym[0] == 'Br' and sort_sym[1] == 'Br':  Mol_SO=-6.55
    
        else:  # for non 3rd row elements, first and second rows
            if charge == 0:
                if sort_sym[0] == 'H' and sort_sym[1] == 'C':  Mol_SO=-0.07
                if sort_sym[0] == 'O' and sort_sym[1] == 'H':  Mol_SO=-0.30
                if sort_sym[0] == 'O' and sort_sym[1] == 'N':  Mol_SO=-0.27
                if sort_sym[0] == 'O' and sort_sym[1] == 'Cl':  Mol_SO=-0.61
                if sort_sym[0] == 'S' and sort_sym[1] == 'H':  Mol_SO=-1.01
                if sort_sym[0] == 'P' and sort_sym[1] == 'O':  Mol_SO=-0.53
                if sort_sym[0] == 'Si' and sort_sym[1] == 'H':  Mol_SO=-0.34
    
            if charge == -1:
                if sort_sym[0] == 'N' and sort_sym[1] == 'H':  Mol_SO=-0.12
                if sort_sym[0] == 'P' and sort_sym[1] == 'H':  Mol_SO=-0.45
                if sort_sym[0] == 'O' and sort_sym[1] == 'O':  Mol_SO=-0.34
                if sort_sym[0] == 'S' and sort_sym[1] == 'S':  Mol_SO=-1.12
    
            if charge == +1:
                if sort_sym[0] == 'H' and sort_sym[1] == 'F':  Mol_SO=-0.62
                if sort_sym[0] == 'P' and sort_sym[1] == 'H':  Mol_SO=-0.67
                if sort_sym[0] == 'H' and sort_sym[1] == 'Cl':  Mol_SO=-1.60
                if sort_sym[0] == 'N' and sort_sym[1] == 'N':  Mol_SO=-0.17
                if sort_sym[0] == 'O' and sort_sym[1] == 'O':  Mol_SO=-0.43
                if sort_sym[0] == 'P' and sort_sym[1] == 'P':  Mol_SO=-0.57
                if sort_sym[0] == 'S' and sort_sym[1] == 'S':  Mol_SO=-1.25
                if sort_sym[0] == 'Cl' and sort_sym[1] == 'Cl':  Mol_SO=-1.77
                if sort_sym[0] == 'F' and sort_sym[1] == 'Cl':  Mol_SO=-1.60
    
    return(Mol_SO)



def nanb(at):
    """
    Returns the number of alpha and beta electron in the element.

            Parameters:
                    at (char): Symbol of the element

            Returns:
                    nanb (list, int): Number of alpha electrons, number of beta electrons in the atom 
    """

    na_dict_1 = {"H" : 1,"He" : 1,"Li" : 1,"Be" : 1,"B" : 2,"C" : 3,"N" : 4,"O" : 4,"F" : 4,"Ne" : 4,"Na" : 1,"Mg" : 1,"Al" : 2,"Si" : 3,"P" : 4,"S" : 4,"Cl" : 4,"Ar" : 4,"K" : 1,"Ca" : 1,\
        "Ga" : 2,"Ge" : 3,"As" : 4,"Se" : 4,"Br" : 4,"Kr" : 4,"Fe" : 10,"I" : 4}
    nb_dict_1 = {"H": 0,"He": 1, "Li" : 0,"Be": 1,"B": 1,"C": 1,"N": 1,"O": 2,"F": 3,"Ne": 4,"Na": 0,"Mg": 1,"Al": 1,"Si": 1,"P": 1,"S": 2,"Cl": 3,"Ar": 4,"K": 0,"Ca": 1,"Ga": 1,"Ge": 1,"As": 1,\
        "Se": 2,"Br": 3,"Kr": 4,"Fe": 6,"I": 3,}
    if at in na_dict_1:
        na = na_dict_1[at]
        nb = nb_dict_1[at]
        return(na,nb)
    else:  # equivalent to case default, if at not in na_dict_1, this is printed, revist
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write("Error: unknown element type encoutered in nanb in module_geom.f90:  " + str(at) + " \n")




def get_ctrl(user_method_type):
    """
    Returns the default control variables/values of the chosen method
            Parameters:
                    user_method_type (char): Method chosen by user in pople.inp
                    
            Returns:
                    val (dict): Dictionary of all control variables required for pople run
    """
    val = {}
#### IMPORTANT: use only "true"  or "false" here -- this section is case sensitive  ### THANKS!
    if user_method_type == "g4mp2-xp" or user_method_type == "G4MP2-XP" or user_method_type == "G4MP2-xp" or user_method_type == "g4mp2-XP":

        val = { "method_opt_freq" : "B3LYP/G Grid7 RIJCOSX def2/J GridX9 Printbasis ", 
            "basis_opt_freq" : "GTBAS3",
            "custombasis_opt_freq" : "true",

            "AA" : 9.472 ,
            "BB" : 3.102 ,
            "CC" : 9.741 ,
            "DD" : 2.115 ,
            "ApAp" : 9.769 ,
            "EE" : 2.379 ,

            "String_Opt": "TightOpt",

            "MGGA" : "false",

            "FROZEN_GEOM" : "false",

            "method_ccsdt" : "DLPNO-CCSD(T) RI-MP2 def2-SVP/C RIJK def2/JK TightPNO Printbasis ",
            "basis_ccsdt" : "GTBAS1",
            "custombasis_ccsdt" : "true",
            "DLPNO_CCSDT" : "true",

            "method_mp2_s" : "RI-MP2 def2-SVP/C RIJK def2/JK Printbasis ",
            "basis_mp2_s" : "GTBAS1",
            "custombasis_mp2_s" : "true",

            "method_mp2" :  "RI-MP2 def2-TZVP/C RIJK def2/JK Printbasis ",
            "basis_mp2" : "GTMP2largeXP",
            "custombasis_mp2" : "true",
            "flag_RIMP2" : "true",
            "flag_DLPNOMP2" : "false",

            "method_hf3" : "HF RIJK def2/JK Printbasis ",
            "basis_hf3" : "GFHFB3",
            "custombasis_hf3" : "true",

            "method_hf4" : "HF RIJK def2/JK Printbasis ",
            "basis_hf4" : "GFHFB4",
            "custombasis_hf4" : "true",

            "scalfac" : "0.9854",

            "conv_scf" : "VeryTight",

            "HLCeqZERO" : "false",

            "SOSCF" : "false",
            "SCFDIIS" : "false",

            "SO_3rdrow_mols" : "true",

            "LSHIFT" : "false",

            "optdiis" : "false",

            "HF_CBS_default"     : "true",
            "HF_CBS_orca_23_def2" : "false",
            "HF_CBS_orca_34_def2" : "false",
            "HF_CBS_orca_23_cc"   : "false",
            "HF_CBS_orca_34_cc"   : "false",

            "restart_cc"  : "false",
            "restart_mp2" : "false",
            "restart_hf3" : "false",
            "restart_hf4" : "false"
        }
        
    if user_method_type == "g4mp2" or user_method_type == "G4MP2":
        print("entering ctrl")
        val = { 
        "method_opt_freq"      : "B3LYP/G Grid5 " ,
        "basis_opt_freq"       : "GTBAS3" ,
        "custombasis_opt_freq" : "true" ,

        "AA" : 9.472 ,
        "BB" : 3.102 , 
        "CC" : 9.741 , 
        "DD" : 2.115 , 
        "ApAp" : 9.769 , 
        "EE" : 2.379 ,


        "String_Opt" : "TightOpt" ,
    
        "MGGA" : "false" ,
    
        "FROZEN_GEOM" : "false" ,
    
        "method_ccsdt" : "CCSD(T) " ,
        "basis_ccsdt" : "GTBAS1" ,
        "custombasis_ccsdt" : "true" ,
        "DLPNO_CCSDT" : "false" ,
        "switch_RIMP2_small" : "false" ,
    
        "switch_read_RIMP2_small" : "false" ,
    
        "method_mp2_s" : "MP2 " ,
        "basis_mp2_s" : "GTBAS1" ,
        "custombasis_mp2_s" : "true" ,
    
        "method_mp2" : "MP2 " ,
        "basis_mp2" : "GTMP2largeXP" ,
        "custombasis_mp2" : "true" ,
        "flag_RIMP2" : "false" ,
        "flag_DLPNOMP2" : "false" ,
    
        "method_hf3" : "HF " ,
        "basis_hf3" : "GFHFB3" ,
        "custombasis_hf3" : "true" ,
    
        "method_hf4" : "HF " ,
        "basis_hf4" : "GFHFB4" ,
        "custombasis_hf4" :"true" ,
    
        "scalfac" : "0.9854" ,
    
        "conv_scf" : "VeryTight" ,
    
        "HLCeqZERO" : "false" ,
    
        "SOSCF" : "false" ,
        "SCFDIIS" : "false" ,
    
        "SO_3rdrow_mols" : "true" ,
    
        "LSHIFT" : "false" ,
    
        "optdiis" : "false" ,
    
        "HF_CBS_default"      : "true" ,
        "HF_CBS_orca_23_def2" : "false" ,
        "HF_CBS_orca_34_def2" : "false" ,
        "HF_CBS_orca_23_cc"   : "false" ,
        "HF_CBS_orca_34_cc"   : "false" ,
    
        "restart_cc"  : "false" ,
        "restart_mp2" : "false" ,
        "restart_hf3" : "false" ,
        "restart_hf4" : "false" 
        }


    return(val)



import os, sys, string
import linecache, math
import numpy as np
import datetime , time


from pople import Mol_SO
from pople import At_SO
from pople import principal_coord
from pople import runorca


####### rung4mp2 - S
def rung4mp2(values, start_time_main): 
    """
    Runs g4mp2

            Parameters:
                    values (dict): Values of the control variables mentioned by user in control.inp
                    start_time_main (float) :  Initial start time of the program pople.run()

    """
 
    if values["isatom"] == "true": Nat=1
    os.system("cat inp.xyz")


    if values["isatom"] != "true":
        if values["FROZEN_GEOM"] == "true":
            start_time1 = time.time()
            with open("read_geom_freq.dat", "r") as froz_geom:
                num_l_fg = sum(1 for l in froz_geom)
                Nat = int(linecache.getline("read_geom_freq.dat",1).strip())
                l2_chr_mul = linecache.getline("read_geom_freq.dat",2)
                sym = []
                Rlist = []
                sym_coord = []
                for tmp_l in range(3,Nat+2):    ## check range
                    l1_xyz_1 = linecache.getline("read_geom_freq.dat",tmp_l)
                    l1_lsp1 = l1_xyz_1.split()
                    full_l1 = l1_xyz_1.strip().split()
                    full_l = full_l1[0] + "     "+str(round(float(full_l1[1]),8)) + "     "+str(round(float(full_l1[2]),8)) + "     "+str(round(float(full_l1[3]),8))
                    sym_coord.append(full_l)  ## contains atom and coords
                    sym.append(l1_lsp1[0])
                    Rlist.append(l1_lsp1[1])
                    Rlist.append(l1_lsp1[2])
                    Rlist.append(l1_lsp1[3])
                    conv = np.array(Rlist)
                    R_coord = np.resize(conv,[Nat,3])
                freq =[]
                for tmp_j in range(Nat+3, num_l_fg+1):
                    tmp_line = float(linecache.getline("read_geom_freq.dat",tmp_j).strip())
                    freq.append(tmp_line)

                print("freq check")
                print(freq)

            if Nat > 2:
                Ievals = principal_coord() # check
                linnonlin = "NL"
                print("FROZEN_GEOM", FROZEN_GEOM)
                print("Eigenvalues of Moment of inertia:", Ievals)
                if abs(Ievals[0]) < 1*(10**-10):  ## CHECK esp Ievals[0 or 1]
                    linnonlin = "L"
            if Nat == 2:    linnonlin = "L"

            if linnonlin == "L":    nfreq = 3 * Nat - 5
            if linnonlin == "NL":   nfreq = 3 * Nat - 6


        else:
            start_time1 = time.time()
            print("==check isatom==",values["isatom"])
            runorca.runorca(values["method_opt_freq"], values["basis_opt_freq"], "true", values["custombasis_opt_freq"], "false", values)
            end_time1 = time.time()

            if values["IPss"] != "true" or values["verticalIP"] != "true":
                with open("inp.xyz", "r") as new_i_xyz:
                    num_l_in = sum(1 for l in new_i_xyz)
                    Nat = int(linecache.getline("inp.xyz",1).strip())
                    title = linecache.getline("inp.xyz",2).strip()
                    tl1 = title.split()
                    charge = int(tl1[0])
                    multip = int(tl1[1])
                with open("input.xyz", "r") as new_i_xyz:
                    num_l_in = sum(1 for l in new_i_xyz)
                    Nat = int(linecache.getline("input.xyz",1).strip())
                    title = linecache.getline("input.xyz",2).strip()
                    tl1 = title.split()
                    sym = []
                    Rlist = []
                    sym_coord = []
                    for tmp_l in range(3,num_l_in+1):    ## check range
                        l1_xyz_1 = linecache.getline("input.xyz",tmp_l)
                        l1_lsp1 = l1_xyz_1.split()
                        full_l1 = l1_xyz_1.strip().split()
                        full_l = full_l1[0] + "     "+str(round(float(full_l1[1]),8)) + "     "+str(round(float(full_l1[2]),8)) + "     "+str(round(float(full_l1[3]),8))
                        sym_coord.append(full_l)  ## contains atom and coords
                        sym.append(l1_lsp1[0])
                        Rlist.append(l1_lsp1[1])
                        Rlist.append(l1_lsp1[2])
                        Rlist.append(l1_lsp1[3])
                        conv = np.array(Rlist)
                        R_coord = np.resize(conv,[Nat,3])
                with open("temp.xyz", "w") as t_xyz:
                   e1= linecache.getline("inp.xyz",1).strip()
                   t_xyz.write(e1 + "\n")
                   e2= linecache.getline("inp.xyz",2).strip()
                   t_xyz.write(e2 + "\n")
                   for e3 in range(len(sym_coord)):
                       t_xyz.write(sym_coord[e3] + "\n")
            os.system("mv temp.xyz  inp.xyz")

############################################
            if Nat > 2:
                Ievals = principal_coord() # check
                linnonlin = "NL"
                if abs(Ievals[0]) < 1*(10**-10):  ## CHECK esp Ievals[0 or 1]
                    linnonlin = "L"
            if Nat == 2:    linnonlin = "L"

#############################################
        
            os.system("nfreq=$( grep 'The total number of vibrations considered is ' input.out  | tail -1 | awk '{print $8}' > scr_lc0);  \
                 nfreq=$( cat scr_lc0); grep -A$(( $nfreq+4 )) 'IR SPECTRUM' input.out | tail -$nfreq | awk '{print $2}' > freq.txt")

    ### need to extract nfreq from scr file
            tr = linecache.getline("scr_lc0",1).strip()
            lsp111 = tr.split()
            nfreq = int(lsp111[0])

            if linnonlin == "NL":
                if nfreq > (3*Nat)-6:
                    nfreq = (3*Nat)-6
                    os.system("nfreq=$( cat scr_lc0); grep -A$(( $nfreq+4 )) 'IR SPECTRUM' input.out | tail -$(( "+str(nfreq)+" )) | awk '{print $2}' > freq.txt")

            freq =[]
            with open("freq.txt", "r") as fr:
                num_l_frq = sum(1 for l in fr)
                for tmp_p in range(1,num_l_frq+1):  # check range
                    tmp_val = float(linecache.getline("freq.txt",tmp_p).strip())
                    freq.append(tmp_val)
            print("freq check 2", freq)
            
            
            iopt = 999
            if values["isatom"] != "true":
                os.system("grep 'OPTIMIZATION RUN DONE' input.out | wc | awk '{print $1}' > scr_lc1")
            iopt = int(linecache.getline("scr_lc1",1).strip())
            
            os.system("rm -f input* scr freq.txt")

    ################################################################################################

        if linnonlin == "NL":
            if nfreq != ((3*Nat)-6):
                with open("Thermochemistry.out", "a") as ther_chem:
                    ther_chem.write(" ************************************\n")
                    ther_chem.write(" * WARNING: No. of freq. != 3N - 6 *\n")  # WARNING!!
                    ther_chem.write(" ************************************\n")
       # MANUAL
        #freq = [30.0,324.0, 3433, 4, 4, 4, 5, 5, 6,7 , 8, 8,30.0,324.0, 3433, 4, 4, 4, 5, 5, 6,7 , 8, 8, 1, 2,3] 
        os.system("rm -f freq0.txt")
        with open("freq0.txt", "a") as new_freq0:
            for tmp_i in range(len(freq)):
                new_freq0.write(str(freq[tmp_i]) + "\n")  # check if this is printed in proper format

    if values["isatom"] != "true":
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write(" Optimized atomic coordinates (Angstrom)\n")
            for tmp in range(Nat):  ## CHECK range
                ther_chem.write(str(sym_coord[tmp]) + "\n")  # writes atoms + coord as from geo,xyz
            ther_chem.write("\n")

            ther_chem.write(" Unscaled harmonic wavenumbers (cm^-1)\n")
            tmp = 0
            for tmp in range(nfreq): # check range
                ther_chem.write(str( freq[tmp] ))  
                ther_chem.write("\n")

            ther_chem.write("\n")
            ther_chem.write(" Scaled harmonic wavenumbers (cm^-1)\n")
            tmp = 0
            for tmp in range(nfreq): # check range
                ther_chem.write(str( freq[tmp] * float(values["scalfac"]) ) )  
                ther_chem.write("\n")

            ther_chem.write("\n")
            ther_chem.write(" Scaling factor used:  " + str(values["scalfac"]) + "\n\n")
            end_time2 = time.time()
            ther_chem.write(" * Geometry optimization/frequencies done in              " + str(end_time2 - start_time1) + " s\n")
            ther_chem.write(" ** Elapsed time =              " + str(round(end_time2 -start_time_main,2)) + " s\n")
    else:
        with open("Thermochemistry.out", "a") as ther_chem:
            ther_chem.write(" * Geometry optimization skipped for atom\n\n")
    
    if values["isatom"] != "true":
        if values["FROZEN_GEOM"] != "true":
            if iopt == 0:
                with open("Thermochemistry.out", "a") as ther_chem:
                    ther_chem.write("E R R O R: Geometry Optimization Failed")

# !=== SINGLE POINT CCSD(T)
    if values["DLPNO_CCSDT"] == "true":
        values["switch_DLPNO_CCSDT"] = "false"
# major addition - not in f90 , check if other numbers change
#  if HOF, if not atoms
    with open("inp.xyz", "r") as new_i_xyz:
        linecache.clearcache()
        num_l_in = sum(1 for l in new_i_xyz)
        Nat = int(linecache.getline("inp.xyz",1).strip())
        title = linecache.getline("inp.xyz",2).strip()
        tl1 = title.split()
        charge = int(tl1[0])
        multip = int(tl1[1])
        sym = []
        Rlist = []
        sym_coord = []
        for tmp_l in range(3,num_l_in+1):    ## check range
            l1_xyz_1 = linecache.getline("inp.xyz",tmp_l)
            l1_lsp1 = l1_xyz_1.split()
            #full_l = l1_xyz_1.strip()
            full_l1 = l1_xyz_1.strip().split()
            full_l = full_l1[0] + "     "+str(round(float(full_l1[1]),8)) + "     "+str(round(float(full_l1[2]),8)) + "     "+str(round(float(full_l1[3]),8))
            sym_coord.append(full_l)  ## contains atom and coords
            sym.append(l1_lsp1[0])
            Rlist.append(l1_lsp1[1])
            Rlist.append(l1_lsp1[2])
            Rlist.append(l1_lsp1[3])
            conv = np.array(Rlist)
            R_coord = np.resize(conv,[Nat,3])


    start_time1 = time.time()
    if values["Ntotale"] > 0:
        if Nat == 1:
            if values["basis_ccsdt"]  == "GTBAS1": values["basis_ccsdt"] = 'GTBAS1atm'
        if values["restart_cc"] == "true":
            #charge = charge + 2
            tmp_method = "HF "
            values["restart_check"] = "true"
            values["restart_charge"] = charge + 2
            runorca.runorca(tmp_method, values["basis_ccsdt"], "false", values["custombasis_ccsdt"], "true", values)
            #charge = charge - 2
            values["restart_charge"] = values["restart_charge"] - 2
    
        if values["DLPNO_CCSDT"] == "true":
            values["switch_DLPNO_CCSDT"] = "true"
        
        runorca.runorca(values["method_ccsdt"], values["basis_ccsdt"], "false", values["custombasis_ccsdt"], "true", values)
        end_time1= time.time()

        
        if (values["Ntotale"] == 1) or (values["Ntotale"] - values["Ntotalecore"] <= 1):
            os.system("grep 'Total Energy ' input.out  | awk '{print $4}' > scr_lc2")
#            print(values["Ntotale"], values["Ntotalecore"], "==HF==")
        else:
            os.system("grep 'FINAL SINGLE POINT ENERGY ' input.out  | awk '{print $5}' > scr_lc2")
        #    print(values["Ntotale"], values["Ntotalecore"], "==CC==")
        
        tmp = linecache.getline("scr_lc2",1).strip()
        E_ccsdt = float(tmp)
        print("== CCSD(T) Energy ==", E_ccsdt)
        
        if (values["Ntotale"] == 1) or (values["Ntotale"] - values["Ntotalecore"] <= 1):
            E_mp2 = E_ccsdt
        else:
            os.system("grep 'Initial E(tot)' input.out | awk '{print $4}' > scr_lc20")
            tmp = linecache.getline("scr_lc20",1).strip()
            E_mp2 = float(tmp)
            os.system("rm -f input* scr ")

    else:
        E_ccsdt = 0
        E_mp2 = 0

    if values["G4MP2TM"] == "true":
        values["switch_load_rel_file"] = "true"
        values["switch_guess"] = "true"
        values["ALLELE"] = "true"

        if values["Ntotale"] > 0:
            if (values["Notale"] < 1) or (values["Ntotale"] - values["Ntotalecore"] <= 1 ):
                tmp_method = "HF "
                if Nat == 1:
                    if values["basis_ccsdt"] == "GTBAS1": values["basis_ccsdt"] = "GTBAS1atm"
                    runorca.runorca(tmp_method, values["basis_ccsdt_rel"], "false", values["custombasis_ccsdt"], "true", values)
                    #print(tmp_method, values["basis_ccsdt_rel"], "false", values["custombasis_ccsdt"], "--1--")
                else:  # ASK
                    runorca.runorca(tmp_method, values["basis_ccsdt_rel"], "false", values["custombasis_ccsdt"], "true", values)
                    #print(tmp_method, values["basis_ccsdt_rel"], "false", values["custombasis_ccsdt"], "--2--")
            elif (values["Ntotale"] == 2) or (values["Ntotale"] - values["Ntotalecore"] == 1 ):
                if Nat == 1:
                    if values["basis_ccsdt"] == "GTBAS1": 
                        values["basis_ccsdt"] = "GTBAS1atm"  # check
                        runorca.runorca(values["method_ccsdt_rel"], values["basis_ccsdt_rel"], "false", values["custombasis_ccsdt"], "true", values)
                        #print(tmp_method, values["basis_ccsdt_rel"], "false", values["custombasis_ccsdt"], "--3--")
                    else:
                        runorca.runorca(values["method_ccsdt_rel"], values["basis_ccsdt_rel"], "false", values["custombasis_ccsdt"], "true", values)
                        #print(tmp_method, values["basis_ccsdt_rel"], "false", values["custombasis_ccsdt"], "--4--")
            else:
                if values["restart_cc"] == "true":
                    #charge = charge + 2
                    tmp_method = "HF "
                    values["restart_check"] = "true"
                    values["restart_charge"] = charge + 2
                    runorca.runorca(tmp_method, values["basis_ccsdt"], "false", values["custombasis_ccsdt"], "true", values)
                    #charge = charge - 2
                    values["restart_charge"] = values["restart_charge"] - 2

                runorca.runorca(values["method_ccsdt_rel"], values["basis_ccsdt_rel"], "false", values["custombasis_ccsdt"], "true", values)
                #print(tmp_method, values["basis_ccsdt_rel"], "false", values["custombasis_ccsdt"], "--5--")
            end_time1 = time.time()

            os.system("grep 'FINAL SINGLE POINT ENERGY ' input.out  | awk '{print $5}' > scr_lc3")
            tmp = linecache.getline("scr_lc3",1).strip()
            E_ccsdt_rel = float(tmp)
        else:
            E_ccsdt_rel = 0

        values["switch_load_rel_file"] = "false"
        values["switch_guess"] = "false"
        values["ALLELE"] = "false"
            
        E_rel = E_ccsdt_rel - E_ccsdt

    end_time_n = time.time()
    end_time1 = time.time()
    with open("Thermochemistry.out", "a") as ther_chem:
        ther_chem.write("\n")
        ther_chem.write(" * CCSD(T) done in              " + str(round(end_time1 - start_time1,2)) + " s\n")
        ther_chem.write(" ** Elapsed time =              " + str(round(end_time_n - start_time_main,2)) + " s \n\n")
        
    if values["switch_read_RIMP2_small"] != "true":
        os.system("rm -f input* scr ")
    
    if values["DLPNO_CCSDT"] == "true":
        values["switch_DLPNO_CCSDT"] = "false"

    print("block dlpno ccsd check", values["switch_read_RIMP2_small"], values["DLPNO_CCSDT"], "outside")
## == TESTING ( comm from f90)
    if values["DLPNO_CCSDT"] == "true":
        #print("block dlpno ccsd check", values["switch_read_RIMP2_small"], values["DLPNO_CCSDT"], "inside")
        if values["switch_read_RIMP2_small"] != "true":
            #print( multip , values["Ntotale"], values["Ntotalecore"], "ntotale ntotalecore")
            if (multip != 1) or (values["switch_RIMP2_small"] == "true") :
                start_time1 = time.time()
                #print(values["Ntotale"], values["Ntotalecore"], "ntotale ntotalecore")
                if (values["Ntotale"] == "1") or (values["Ntotale"] - values["Ntotalecore"] <= 1) :
                    tmp_method = "HF "
                    #print("block dlpno ccsd check  1")
                    runorca.runorca(values["method_mp2_s"], values["basis_mp2_s"], "false", values["custombasis_mp2_s"], "true", values)
                else:
                    #print("block dlpno ccsd check  2")
                    runorca.runorca(values["method_mp2_s"], values["basis_mp2_s"], "false", values["custombasis_mp2_s"], "true", values)
                end_time1= time.time()

                if (values["Ntotale"] == 1) or (values["Ntotale"] - values["Ntotalecore"] <= 1) :
                    os.system("grep 'Total Energy ' input.out  | awk '{print $4}' > scr_lc4")
                else:
                    os.system("grep 'FINAL SINGLE POINT ENERGY ' input.out  | awk '{print $5}' > scr_lc4")
                tmp = linecache.getline("scr_lc4",1).strip()
                E_mp2 = float(tmp)
                os.system("rm -f scr ")
                
                end_time_n = time.time()
                with open("Thermochemistry.out", "a") as ther_chem:
                    ther_chem.write(" * MP2/S done (for non-singlets) in   " + str(end_time1 - start_time1) + " s \n")
                    ther_chem.write(" ** Elapsed time = " + str(end_time_n - start_time_main) + " s \n\n")
#MP2 TOTAL ENERGY
    if values["switch_read_RIMP2_small"] == "true":
        if (values["Ntotale"] == 1) or (values["Ntotale"] - values["Ntotalecore"] <= 1):
            os.system("grep 'Total Energy ' input.out  | awk '{print $4}' > scr_lc5")
        else:
            os.system("grep 'MP2 TOTAL ENERGY' input.out  | awk '{print $4}' > scr_lc5")
        tmp = linecache.getline("scr_lc5",1).strip()
        E_mp2 = float(tmp)
    os.system("rm -f input* scr ")

# === SINGLE POINT MP2L
    start_time1 = time.time()  # can this be moved into the if statement?
    if (values["Ntotale"] > 0) and (values["ccsdt_cbs"] != "true") : ## where is ccsdt_cbs declared?
        if values["restart_mp2"] == "true":
            #charge = charge + 2
            tmp_method = "HF "
            values["restart_check"] = "true"
            values["restart_charge"] = charge + 2
            runorca.runorca(tmp_method, values["basis_ccsdt"], "false", values["custombasis_ccsdt"], "true", values)
            #charge = charge - 2
            values["restart_charge"] = values["restart_charge"] - 2

        runorca.runorca(values["method_mp2"], values["basis_mp2"], "false", values["custombasis_mp2"], "true", values)
        end_time1= time.time()
    
        if (values["Ntotale"] == 1) or (values["Ntotale"] - values["Ntotalecore"] <= 1):
            os.system("grep 'Total Energy ' input.out  | awk '{print $4}' > scr_lc6")
        else:
            os.system("grep 'FINAL SINGLE POINT ENERGY ' input.out  | awk '{print $5}' > scr_lc6")
        tmp = linecache.getline("scr_lc6",1).strip()
        E_mp2L = float(tmp)

        if (values["Ntotale"] == 1) or (values["Ntotale"] - values["Ntotalecore"] <= 1):
            E_hfL = E_mp2L
            #print("=== E_hfL === 00", E_hfL, E_mp2L, values["Ntotale"], values["Ntotalecore"])
        else:
            if values["flag_RIMP2"] == "true":
                os.system("grep 'RI-MP2 CORRELATION ENERGY:' input.out | awk '{print $4}' > scr_lc7")
            elif values["flag_DLPNOMP2"] == "true":
                os.system("grep 'DLPNO-MP2 CORRELATION ENERGY:' input.out | awk '{print $4}' > scr_lc7")
            else:
                os.system("grep 'MP2 CORRELATION ENERGY ' input.out | awk '{print $5}' > scr_lc7")
            tmp = linecache.getline("scr_lc7",1).strip()
            E_hfL = float(tmp)

            #print("=== E_hfL ===", E_hfL, E_mp2L, values["Ntotale"], values["Ntotalecore"])
            E_hfL = E_mp2L - E_hfL

        os.system("rm -f input* scr ")
# VERIFY - indentation issues, which else this goes to??? revisit
    else:
        E_hfL= 0
        E_mp2L = 0

    end_time_n = time.time()
    with open("Thermochemistry.out", "a") as ther_chem:
        ther_chem.write(" * MP2/L done in                " + str(round(end_time1 - start_time1,2)) + " s \n")
        ther_chem.write(" ** Elapsed time =              " + str(round(end_time_n - start_time_main,2)) + " s \n\n")

#== SINGLE POINT HF/VTZ
    start_time1 = time.time()
    if (values["Ntotale"] > 0) and (values["ccsdt_cbs"] != "true"):
        if values["restart_hf3"] == "true":
            #charge = charge + 2
            tmp_method = "HF "
            values["restart_check"] = "true"
            values["restart_charge"] = charge + 2
            runorca.runorca(tmp_method, values["basis_ccsdt"], "false", values["custombasis_ccsdt"], "true", values)
            #charge = charge - 2
            values["restart_charge"] = values["restart_charge"] - 2

        runorca.runorca(values["method_hf3"], values["basis_hf3"], "false", values["custombasis_hf3"], "false", values)
        end_time1 = time.time()

        os.system("grep 'FINAL SINGLE POINT ENERGY ' input.out  | awk '{print $5}' > scr_lc8")
        tmp = linecache.getline("scr_lc8",1).strip()
        E_hfT = float(tmp)
        
        os.system("rm -f input* scr ")
    else:
        E_hfT = 0
    
    end_time_n = time.time()
    with open("Thermochemistry.out", "a") as ther_chem:
        ther_chem.write(" * HF/VTZ done in              " + str(round(end_time1 - start_time1,2)) + " s\n")
        ther_chem.write(" ** Elapsed time =              " + str(round(end_time_n - start_time_main,2)) + " s\n\n")

# == SINGLE POINT HF/VQZ
    start_time1 = time.time()
    if (values["Ntotale"] > 0 ) and (values["ccsdt_cbs"] != "true"):
        if values["restart_hf4"] == "true":
            #charge = charge + 2
            tmp_method = "HF "
            values["restart_check"] = "true"
            values["restart_charge"] = charge + 2
            runorca.runorca(tmp_method, values["basis_ccsdt"], "false", values["custombasis_ccsdt"], "true", values)
            #charge = charge - 2
            values["restart_charge"] = values["restart_charge"] - 2

        runorca.runorca(values["method_hf4"], values["basis_hf4"], "false", values["custombasis_hf4"], "false", values)
        end_time1 = time.time()

        os.system("grep 'FINAL SINGLE POINT ENERGY ' input.out  | awk '{print $5}' > scr_lc9")
        tmp = linecache.getline("scr_lc9",1).strip()
        E_hfQ = float(tmp)
        
        os.system("rm -f input* scr ")
    else:
        E_hfQ = 0
    
    end_time_n = time.time()
    with open("Thermochemistry.out", "a") as ther_chem:
        ther_chem.write(" * HF/VQZ done in              " + str(round(end_time1 - start_time1,2)) + " s\n")
        ther_chem.write(" ** Elapsed time =              " + str(round(end_time_n  - start_time_main,2)) + " s\n\n")

# Thermal
    kB = 1.38064852000000*(10**-23) #kB = 1.38064852000000d-23
    h = 6.626070040*(10**-34)    #6.626070040d-34
    Ry = 10973731.568508  #10973731.568508d0
    c = 299792458.0    #299792458d0
    T = 298.15  #298.15d0 
    e = 1.60217648700000*(10**-19)   #1.60217648700000d-19
    N_avo = 6.02214179000000*(10**+23)  #6.02214179000000d+23
    au2j = 2 * Ry * h * c

    au2ev  = 2 * Ry * h * c          / e
    au2kjm = 2 * Ry * h * c * N_avo / 1000.0
    au2kcm = 2 * Ry * h * c * N_avo / 4184.0
    kcm2au = 1 / au2kcm

    j2au   = 1 / au2j
    au2cmi = 2 * Ry / 100.0
    cmi2au = 1 / au2cmi
    Theta = []
    if values["isatom"] != "true":
        for tmp_i in range(len(freq)):
            freq[tmp_i] = freq[tmp_i] * float(values["scalfac"])
            Theta_new = c * 100.0 *  h * freq[tmp_i] / kB
            Theta.append(Theta_new)
        dE_vib = 0
        for tmp in range(nfreq):  # check range
            dE_vib = dE_vib + kB * Theta[tmp] * j2au * ( 0.5 + 1 / ( math.exp(Theta[tmp]/T) - 1) )  ## check
        ZPVE = 0.5 * sum(freq) * cmi2au
        if linnonlin == "L": 
            dE_rot = kB * T * j2au
        elif linnonlin == "NL":
            dE_rot = kB * T * j2au * (3.0/2.0)
        else:
            with open("Thermochemistry.out", "a") as ther_chem:
                ther_chem.write("ERROR: unknown entry for linear/nonlinear")
    else:
        ZPVE    = 0
        dE_vib  = 0
        dE_rot  = 0

    dE_tra = kB * T * j2au * (3.0/2.0)
    Ethermal = dE_vib + dE_rot + dE_tra
    print("Vib Rot Tra Tot = ",dE_vib,dE_rot,dE_tra,Ethermal)

#  !=== SO
    HLC_SO = 0
    if values["isatom"] != "true":
        HLC_SO = Mol_SO(Nat, multip, charge, sym, values["SO_3rdrow_mols"])
        #print(HLC_SO)
    else:
        HLC_SO = At_SO(sym[0], charge)
    
    #print("==check near HLC ",values["isatom"],Nat)
    if values["HLCeqZERO"] == "true":
        HLC = 0
    else:
        if values["isatom"] != "true":
            if values["Na"] == values["Nb"]:
                #print("HLC atoms-1:",values["Na"],values["Nb"])
                HLC = - values["AA"] * values["Nb"]
            else:
                #print("HLC atoms-2:",values["Na"],values["Nb"])
                HLC = - values["ApAp"] * values["Nb"] - values["BB"] * (values["Na"]- values["Nb"])   # check
        else:
             #print("HLC atoms-3:",values["Na"],values["Nb"])
             HLC = - (values["CC"] * values["Nb"] ) - (values["DD"] * (values["Na"] - values["Nb"]) )  # + At_SO(sym(1), charge)
        
        if values["isatom"] == "true":
            if sym[0] == "Be" and charge == 0 : HLC = - values["EE"]
            if sym[0] == "Mg" and charge == 0 : HLC = - values["EE"]
            if sym[0] == "Ca" and charge == 0 : HLC = - values["EE"]
            if sym[0] == "Li" and charge == -1 : HLC = - values["EE"]
            if sym[0] == "Na" and charge == -1 : HLC = - values["EE"]
            if sym[0] == "K" and charge == -1 : HLC = - values["EE"]
        elif Nat == 2:
            if charge == 0:
                if sym[0] == "Li" and sym[1] == "Li": HLC = - values["EE"]
                if sym[0] == "Na" and sym[1] == "Na": HLC = - values["EE"]
                if sym[0] == "K" and sym[1] == "K": HLC   = - values["EE"]
                if sym[0] == "Li" and sym[1] == "Na": HLC = - values["EE"]
                if sym[0] == "Na" and sym[1] == "Li": HLC = - values["EE"]
                # if ( (trim(sym(1)) .eq. 'Be') .and. (trim(sym(2)) .eq. 'H' ) ) HLC = -EE
                # if ( (trim(sym(1)) .eq. 'H' ) .and. (trim(sym(2)) .eq. 'Be') ) HLC = -EE

    HLC  = HLC / 1000.0
    HLC_SO = HLC_SO / 1000.0
    values["HLC0"] = HLC

# CBS extrapolation hard-coded for HF (AVQZ, AV5Z)
    if values["HF_CBS_default"] == "true": E_HF_limit = (E_hfQ - E_hfT * math.exp(-1.63)) / (1 - math.exp(-1.63))
    if values["HF_CBS_orca_23_def2"] == "true": E_HF_limit = (E_hfQ - E_hfT * math.exp(-10.39*(math.sqrt(3.0)- math.sqrt(2.0)))) / (1 - math.exp(-10.39*(math.sqrt(3.0)-math.sqrt(2.0))))
    if values["HF_CBS_orca_34_def2"] == "true": E_HF_limit = (E_hfQ - E_hfT * math.exp(-7.88*( math.sqrt(4.0)- math.sqrt(3.0)))) /  (1 - math.exp(-7.88*(math.sqrt(4.0)-math.sqrt(3.0))))
    if values["HF_CBS_orca_23_cc"] == "true":   E_HF_limit = (E_hfQ - E_hfT * math.exp(-4.42*( math.sqrt(3.0)- math.sqrt(2.0)))) / (1 - math.exp(-4.42* (math.sqrt(3.0)-math.sqrt(2.0))))
    if values["HF_CBS_orca_34_cc"] == "true":   E_HF_limit = (E_hfQ - E_hfT * math.exp(-5.46*( math.sqrt(4.0)- math.sqrt(3.0)))) / (1 - math.exp(-5.46* (math.sqrt(4.0)-math.sqrt(3.0))))
    
    E_HF_CBS   = E_HF_limit - E_hfL
    #print(E_mp2L,E_mp2)
    E_MP2_CBS  = E_mp2L - E_mp2
    if values["noHLC"] == "true": HLC = 0
    
    if values["ccsdt_cbs"] == "true":
        E_HF_CBS = 0
        E_MP2_CBS = 0
        eps0 = E_ccsdt +  HLC + HLC_SO
    else:
        eps0 = E_ccsdt + E_HF_CBS + E_MP2_CBS + HLC + HLC_SO
    
    if values["G4MP2TM"] == "true":
        eps0 = eps0 + E_rel

    values["U0"] = eps0 + ZPVE
    values["UT"] = eps0 + Ethermal
    values["HT"] = eps0 + Ethermal + kB * T * j2au

    if values["IPss"] == "true":   ## post check reduce all indices by 1
        print("-----here---")
        values["IPbreakdown2"] = [ E_ccsdt, E_HF_CBS, E_MP2_CBS, HLC,HLC_SO,ZPVE  ]
        values["EAbreakdown2"] = [ E_ccsdt, E_HF_CBS, E_MP2_CBS,HLC,HLC_SO,ZPVE  ]
        values["PAbreakdown2"] = [ E_ccsdt, E_HF_CBS, E_MP2_CBS, HLC, HLC_SO, ZPVE, (dE_vib - ZPVE) , dE_rot, dE_tra , (kB * T * j2au) ]
    else:
        values["IPbreakdown1"] = [ E_ccsdt, E_HF_CBS, E_MP2_CBS, HLC, HLC_SO, ZPVE ]
        values["EAbreakdown1"] = [ E_ccsdt, E_HF_CBS, E_MP2_CBS, HLC, HLC_SO, ZPVE ]
        values["PAbreakdown1"] = [ E_ccsdt, E_HF_CBS, E_MP2_CBS, HLC, HLC_SO, ZPVE, (dE_vib - ZPVE), dE_rot, dE_tra, (kB * T * j2au)  ]

    with open("Thermochemistry.out", "a") as ther_chem:
        ther_chem.write("           Temperature=     " + str(round(T,6)) +  "                   Pressure=       1.000000 \n")
        ther_chem.write("                E(ZPE)=       " +str(round(ZPVE,6)) + "             E(Thermal)=       " + str(round(Ethermal,6)) + "\n")  # revisit
        if values["G4MP2TM"] == "true":
            ther_chem.write("            E(CCSD(T),rel)=       "  + str(round(E_ccsdt_rel,6)) + "\n")
        ther_chem.write("                   HLC=      " + str(round(HLC,6)) + "                     SO=       " + str(round(HLC_SO,6)) + "  \n")
        ther_chem.write("            E(CCSD(T))=     " + str(round(E_ccsdt,6))  + "\n")
        ther_chem.write("               DE(MP2)=      "  + str(round(E_MP2_CBS,6)) + "                 DE(HF)=      " + str(round(E_HF_CBS,6))  + "\n")
        ther_chem.write("            G4MP2(0 K)=     "  + str(round(values["U0"],6))  + "           G4MP2 Energy=     " + str(round(values["UT"],6)) + "\n")
        ther_chem.write("        G4MP2 Enthalpy=     " + str(round(values["HT"],6)) + "      G4MP2 Free Energy=       0.000000 \n\n")  # verify
        
        end_time_n = time.time()
        ther_chem.write(" * G4(MP2) done\n")
        ther_chem.write(" ** Elapsed time =              "  + str(round(end_time_n - start_time_main,2))  + "  s \n\n")
        
    os.system("rm -f scr scrd")

####### rung4mp2 - E
